unit QTM_Complex;
interface
uses Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Pesgvcl, StdCtrls, ExtCtrls, QTM_Decl, PEgrpapi, CheckLst, Menus, Math;

type
  T22Matrix_C = array[1..2,1..2] of TComplex;
  T2Vector_C = array[1..2] of TComplex;

function complex(a,b : Double): TComplex;
function abs_c(a : TComplex): Double;
function arg_c(a : TComplex): Double;
function conjugate_c(a : TComplex): TComplex;
function plus_c(a,b : TComplex): TComplex;
function minus_c(a,b : TComplex): TComplex;
function times_c(a,b : TComplex): TComplex;
function inverse_c(a : TComplex): TComplex; overload;
function inverse_c(a : TComplex; var Errorcode: Byte): TComplex;  overload;
function sqrt_c(a : TComplex): TComplex;
function div_c(a,b : TComplex): TComplex; overload;
function div_c(a,b : TComplex; var Errorcode: Byte): TComplex;  overload;
function exp_c(a : TComplex): TComplex;
function ln_c(a : TComplex): TComplex; overload;
function ln_c(a : TComplex; var Errorcode: Byte): TComplex;  Overload;
function cos_c(a : TComplex): TComplex;
function sin_c(a : TComplex): TComplex;
function tan_c(a : TComplex): TComplex;
function sqr_c(a : TComplex): TComplex;
function cosh_c(a : TComplex): TComplex;
function sinh_c(a : TComplex): TComplex;
function Power_c(a : TComplex; p : integer): TComplex;

function matrix_times_vector_c(a : T22Matrix_C; u : T2Vector_C):T2Vector_C;
function powers_prod_c(a1 : TComplex; p1 : integer;
                     a2 : TComplex; p2: integer;
                     a3 : TComplex; p3 : integer;
                     a4 : TComplex; p4 : integer;
                     a5 : TComplex; p5 : integer) : TComplex;

implementation
uses QTM_Main;

function complex(a,b: Double): TComplex;
begin
  complex.real := a; complex.imag := b;
end;

function abs_c(a: TComplex): Double;
begin
  abs_c := sqrt(sqr(a.real)+sqr(a.imag));
end;

function arg_c(a: TComplex): Double;
begin
  if (a.real = 0) and (a.imag = 0) then arg_c := 0;
  if (a.real = 0) and (a.imag > 0) then arg_c := Pi/2;
  if (a.real = 0) and (a.imag < 0) then arg_c := -Pi/2;
  if (a.real <> 0) then arg_c := ArcTan2(a.imag, a.real);
end;

function conjugate_c(a : TComplex): TComplex;
begin
  conjugate_c.real := a.real;
  conjugate_c.imag := -a.imag;
end;

function plus_c(a,b : TComplex): TComplex;
begin
  plus_c.real := a.real + b.real;
  plus_c.imag := a.imag + b.imag ;
end;

function minus_c(a,b : TComplex): TComplex;
begin
  minus_c.real := a.real - b.real;
  minus_c.imag := a.imag - b.imag;
end;

function times_c(a,b : TComplex): TComplex;
begin
  times_c.real := a.real*b.real - a.imag*b.imag;
  times_c.imag := a.real*b.imag + a.imag*b.real;
end;

function div_c(a,b : TComplex): TComplex; overload;
var Errorcode: Byte;
begin
  result := div_c(a,b, Errorcode);
end;

function div_c(a,b : TComplex; var Errorcode: Byte): TComplex;  overload;
begin
  if abs_c(b) > 0 then div_c := times_c(a,inverse_c(b,Errorcode))
  else begin
    MessageDlg('div_c: Division by zero', mtInformation, [mbOK], 0);
    Errorcode := Errorcode or E_Complex_failed;
  end;
end;

function inverse_c(a : TComplex): TComplex; overload;
var Errorcode: Byte;
begin
  result := inverse_c(a, Errorcode)
end;


function inverse_c(a : TComplex; var Errorcode: Byte): TComplex;  overload;
begin
  if abs_c(a) <> 0 then begin
    inverse_c.real :=  a.real /sqr(abs_c(a));
    inverse_c.imag := -a.imag /sqr(abs_c(a));
  end else begin
    MessageDlg('inverse_c: Division by zero', mtInformation, [mbOK], 0);
    Errorcode := Errorcode or E_Complex_failed;
  end;
end;

function  exp_c(a : TComplex): TComplex;
begin
  exp_c.real := exp(a.real) * cos(a.imag);
  exp_c.imag := exp(a.real) * sin(a.imag);
end;

function ln_c(a : TComplex): TComplex; overload;
var Errorcode: Byte;
begin
  result := ln_c(a, Errorcode)
end;

function ln_c(a : TComplex; var Errorcode: Byte): TComplex;  Overload;
begin
  if abs_c(a) = 0 then begin
    if MessageDlg('ln_c: ln of zero, exit ?', mtInformation, [mbYES, mbNo],0) = mrYes then begin
      Errorcode := Errorcode or E_Complex_failed;
      MessageDlg('There appears to be problem with the configuration', mtInformation, [mbYes, mbNo],0);
    end;
  end;
  if abs_c(a) <> 0 then begin
    ln_c.real := ln(abs_c(a));
    ln_c.imag := ArcTan2(a.imag, a.real);
  end;
end;

function cos_c(a : TComplex): TComplex;
begin
  cos_c.real :=  cos(a.real) * cosh(a.imag);
  cos_c.imag := -sin(a.real) * sinh(a.imag);
end;

function sin_c(a : TComplex): TComplex;
begin
  sin_c.real :=  sin(a.real) * cosh(a.imag);
  sin_c.imag :=  cos(a.real) * sinh(a.imag);
end;

function tan_c(a : TComplex): TComplex;
begin
  if (cos(2*a.real) + cosh(2*a.imag)) <> 0 then begin
    tan_c.real := sin( 2*a.real) / (cos(2*a.real) + cosh(2*a.imag));
    tan_c.imag := sinh(2*a.imag) / (cos(2*a.real) + cosh(2*a.imag));
  end else Messagedlg('tangent is infinite', mtinformation, [mbOK], 0);
end;


function matrix_times_vector_c(a : T22Matrix_C; u : T2Vector_C):T2Vector_C;
begin
  matrix_times_vector_c[1] := plus_c(times_c(a[1,1],u[1]),times_c(a[1,2],u[2]));
  matrix_times_vector_c[2] := plus_c(times_c(a[2,1],u[1]),times_c(a[2,2],u[2]));
end;

function sqrt_c(a : TComplex): TComplex;
begin
  sqrt_c.real := sqrt(abs_c(a)) * cos(0.5*Arg_c(a));
  sqrt_c.imag := sqrt(abs_c(a)) * sin(0.5*Arg_c(a));
end;

function sqr_c(a : TComplex): TComplex;
begin
  sqr_c.real := sqr(a.real) - sqr(a.imag);
  sqr_c.imag := 2 * a.real * a.imag;
end;

function cosh_c(a : TComplex): TComplex;
begin
  cosh_c.real :=  cosh(a.real) * cos(a.imag);
  cosh_c.imag :=  sinh(a.real) * sin(a.imag);
end;

function sinh_c(a : TComplex): TComplex;
begin
  sinh_c.real :=  sinh(a.real) * cos(a.imag);
  sinh_c.imag :=  cosh(a.real) * sin(a.imag);
end;

function Power_c(a : TComplex; p : integer): TComplex;
var pc : TComplex;
begin
  if (a.real = 0) and (a.imag = 0) then Power_c := Complex(0, 0)
  else begin
    pc := Complex(p, 0);
    Power_c := exp_c(Times_c(pc, ln_c(a)));
  end;
end;

function powers_prod_c(a1 : TComplex; p1 : integer;
                     a2 : TComplex; p2: integer;
                     a3 : TComplex; p3 : integer;
                     a4 : TComplex; p4 : integer;
                     a5 : TComplex; p5 : integer) : TComplex;
var a1p, a2p, a3p, a4p, a5p : TComplex;
begin
  a1p := Power_c(a1, p1);
  a2p := Power_c(a2, p2);
  a3p := Power_c(a3, p3);
  a4p := Power_c(a4, p4);
  a5p := Power_c(a5, p5);
  powers_prod_c := Times_c(a1p,
                    Times_c(a2p,
                      Times_c(a3p,
                       Times_c(a4p, a5p))));
end;

end.






