unit QTM_DfDG_vs_actpar;
interface
uses Windows,Messages,SysUtils,Variants,Classes,Graphics,Controls,Forms,Dialogs,QTM_Decl,StdCtrls,Pesgvcl,Pegrpapi;
type
  TDfDG_vs_actpar = class(TForm)
    df_Graph: PESGraph; dG_Graph: PESGraph;
    lower_Label: TLabel; upper_Label: TLabel; Limits_Label: TLabel;
    low_limit_Show: TEdit; up_limit_Show: TEdit;
    Suggest_Btn: TButton;
    Actpar_Label: TLabel;
    n_label: TLabel;
    procedure Read_Panel_Format;
    procedure Write_Panel_Format;
    procedure Status_Read;
    procedure Update_Values;
    procedure Simulate;
    procedure Update_Graphs;
    procedure Arrange;
    procedure Suggest_limits;

    procedure FormResize(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormDeactivate(Sender: TObject);
    procedure Suggest_BtnClick(Sender: TObject);
    procedure low_limit_ShowKeyPress(Sender: TObject; var Key: Char);
    procedure up_limit_ShowKeyPress(Sender: TObject; var Key: Char);
  public
    low_limit,up_limit : Double;
    B_NoClick : Boolean;
    df_simul,dG_simul : array[1..nhm,1..n_inpol_max] of Double;
    x_arr : array[1..n_inpol_max] of Double;
    n_inpol : Integer;
  end;

var DfDG_vs_actpar: TDfDG_vs_actpar;

implementation
uses QTM_Main,QTM_MIO,QTM_SLA,QTM_Perturbation;
{$R *.dfm}

procedure TDfDG_vs_actpar.FormResize(Sender: TObject); begin if B_NoClick then Exit; Arrange; end;
procedure TDfDG_vs_actpar.FormCreate(Sender: TObject);
begin
  Status_Read
end;
procedure TDfDG_vs_actpar.FormClose(Sender: TObject; var Action: TCloseAction); begin Write_Panel_Format; end;
procedure TDfDG_vs_actpar.FormDeactivate(Sender: TObject); begin Write_Panel_Format; end;

procedure TDfDG_vs_actpar.FormShow(Sender: TObject);
begin
  B_NoClick := True;
  Read_Panel_Format; n_label.Visible := False;
  Suggest_limits;
  Update_Values; Simulate; Update_Graphs;
  B_NoClick := False;
  Arrange;
end;

procedure TDfDG_vs_actpar.Read_Panel_Format;
begin
  Top := Ini.ReadInteger('DfDG_vs_actpar','Top',0);
  Height := Ini.ReadInteger('DfDG_vs_actpar','Height',200);
  Left := Ini.ReadInteger('DfDG_vs_actpar','Left',0);
  Width := Ini.ReadInteger('DfDG_vs_actpar','Width',200);
end;

procedure TDfDG_vs_actpar.Write_Panel_Format;
begin
  if B_NoClick then Exit;
  Ini.WriteInteger('DfDG_vs_actpar','Top',Top);
  Ini.WriteInteger('DfDG_vs_actpar','Height',Height);
  Ini.WriteInteger('DfDG_vs_actpar','Left',Left);
  Ini.WriteInteger('DfDG_vs_actpar','Width',Width); B_Changed := True;
end;

procedure TDfDG_vs_actpar.Status_Read;
begin
  low_limit := QTMFile.ReadFloat('DfDG_vs_actpar','low_limit',1);
  up_limit := QTMFile.ReadFloat('DfDG_vs_actpar','up_limit',10);
end;

procedure TDfDG_vs_actpar.Suggest_BtnClick(Sender: TObject);
begin
  Suggest_limits; Update_Values; Simulate; Update_Graphs;
end;

procedure TDfDG_vs_actpar.Update_Values;
begin
  low_limit_Show.Text := FloatToStr(low_limit);
  up_limit_Show.Text := FloatToStr(up_limit);
  actpar_Label.Caption := 'Active Parameter : ' +  MIO.Graph_labels(i_rbtn);
end;

procedure TDfDG_vs_actpar.Simulate;
var Errorcode: Byte;
    x_old,dummy : Double;
    i,n : integer;
begin
  Screen.Cursor := crHourglass;
  n_inpol := 200;
  MIO.Pars_to_x(x_old);
  for n := 1 to nhm do begin
    if B_fit_inc[n] then begin
    n_r := n_arr[n];
      n_label.Caption := IntToStr(n);
      n_label.Visible := True; application.processmessages;
      for i := 1 to n_inpol do begin
        x_arr[i] := low_limit + (i-1)/(n_inpol-1)*(up_limit - low_limit);
        SLA.x_to_Pars(x_arr[i]);
//        MIO.User_Pars_to_GPars_JpJpp(Pars_JpJpp,Pars);
        if not(Errorcode = 0) then Exit;
        case i_model of
          j_model_AMF : SLA.dfstar_from_Z(df_simul[n,i],dG_simul[n,i],dummy,dummy,Errorcode);
          j_model_PE3 : Pert.dfstar_Pe3(df_simul[n,i],dG_simul[n,i],dummy,dummy,Errorcode);
          j_model_PE5 : Pert.dfstar_Pe5(df_simul[n,i],dG_simul[n,i],dummy,dummy,Errorcode);
        end;
      end;
    end;
  end;
  SLA.x_to_Pars(x_old);
  Screen.Cursor := crdefault; n_label.Visible := False; application.processmessages;
end;

procedure TDfDG_vs_actpar.Update_Graphs;
var n,s,p,ig : integer;
    graph : PESGraph;
    n_poi_arr : array[1..nhm] of Integer;
begin
  for ig := 1 to 2 do begin
    case ig of
      1 : graph := df_Graph;
      2 : graph := dG_Graph;
    end;
    with graph do begin
      PEactions := sgreset; CursorPromptTracking := True; CursorPromptStyle := sgXandYValue;
      AllowZooming := sgHorzPlusVertZooming; FontSize := sgLarge; ShadowColor := clNone;
      DataShadows := sgNo3D; GridLineControl := sgNoGrid; NullDataValueX := GapV; NullDataValueY := GapV;
      MainTitleFont := 'Arial'; SubTitleFont := 'Arial'; Maintitle := ''; Subtitle := '';
      nhar_fit := 0;
      for n := 1 to nhm do begin
        if B_fit_inc[n] then begin inc(nhar_fit); n_poi_arr[nhar_fit] := n; end;
      end;
      Points := n_inpol; Subsets := nhar_fit;
      XaxisLabel := MIO.Graph_labels(i_rbtn);
      case ig of
        1 : YaxisLabel := M.df_Graph.YaxisLabel;
        2 : YaxisLabel := M.dG_Graph.YaxisLabel;
      end;
      for s := 1 to Subsets do begin
        n := n_poi_arr[s];
        SubsetColors[s-1] := QTZ_Colors[s];
        SubsetLabels[s-1] := IntToStr(n);
        SubsetLineTypes[s-1] := PELT_ThinSOLID;
        for p := 1 to n_inpol do begin
          XData[s-1,p-1] := x_arr[p];
          case ig of
            1 : begin
              case i_yaxis of
                1,5 : YData[s-1,p-1] := df_simul[n,p]                  ;
                2,6 : YData[s-1,p-1] := df_simul[n,p] / n_arr[n]       ;
                3   : YData[s-1,p-1] := df_simul[n,p] / sqrt(n_arr[n]) ;
                4   : YData[s-1,p-1] := df_simul[n,p] * n_arr[n]       ;
              end;
              if df_simul[n,p] = GapV then YData[s-1,p-1] := GapV;
            end;
            2 : begin
              case i_yaxis of
                1 : YData[s-1,p-1] := dG_simul[n,p];
                2 : YData[s-1,p-1] := dG_simul[n,p] / n_arr[n];
                3 : YData[s-1,p-1] := dG_simul[n,p] / sqrt(n_arr[n]);
                4 : YData[s-1,p-1] := dG_simul[n,p] * n_arr[n];
                5 : if (df_simul[n,p] <> 0) then YData[s-1,p-1] := dG_simul[n,p]/ -df_simul[n,p]
                    else YData[s-1,p-1] := GapV;
                6 : YData[s-1,p-1] := 1e6 * 2 * dG_simul[n,p] / (f_fund * n_arr[n]) ;
              end;
              if dG_simul[n,p] = GapV then YData[s-1,p-1] := GapV;
            end;
          end;
        end;  // for p
      end; // for s
      PEactions := sgReInitAndReset;
    end;  //with graph
  end; //ig
end;

procedure TDfDG_vs_actpar.Arrange;
begin
  df_Graph.Width := DfDG_vs_actpar.Width - df_Graph.Left - 40;
  dG_Graph.Width := df_Graph.Width;
  df_Graph.Left := 1;
  dG_Graph.Left := df_Graph.Left;
  df_Graph.Height := round((DfDG_vs_actpar.Height - df_Graph.Top) / 2.5);
  dG_Graph.Height := df_Graph.Height;
  dG_Graph.Top := df_Graph.Top + df_Graph.Height + 1;
end;

procedure TDfDG_vs_actpar.Suggest_Limits;
begin
  with Pars do begin
    case i_rBtn of
      t_e_ID       : begin low_limit := t_e / 2; up_limit := t_e * 2; end;
      t_f_ID       : begin low_limit := t_f / 2; up_limit := t_f * 2; end;
      rho_e_ID     : begin low_limit := rho_e / 2; up_limit := rho_e * 2; end;
      rho_f_ID     : begin low_limit := rho_f / 2; up_limit := rho_f * 2; end;
      rho_b_ID     : begin low_limit := rho_b / 2; up_limit := rho_b * 2; end;
      Jp_e_ID      : begin low_limit := Jp_e / 2; up_limit := Jp_e * 2; end;
      Jp_f_ID      : begin low_limit := Jp_f / 2; up_limit := Jp_f * 2; end;
      Jp_b_ID      : begin low_limit := Jp_b / 2; up_limit := Jp_b * 2; end;
      Jp_exp_e_ID  : begin low_limit := Jp_exp_e / 2; up_limit := Jp_exp_e * 2; end;
      Jp_exp_f_ID  : begin low_limit := Jp_exp_f / 2; up_limit := Jp_exp_f * 2; end;
      Jp_exp_b_ID  : begin low_limit := Jp_exp_b / 2; up_limit := Jp_exp_b * 2; end;
      Jpp_e_ID     : begin low_limit := Jpp_e / 2;   up_limit := Jpp_e * 2; end;
      Jpp_f_ID     : begin low_limit := Jpp_f / 2;   up_limit := Jpp_f * 2; end;
      Jpp_b_ID     : begin low_limit := Jpp_b / 2;   up_limit := Jpp_b * 2; end;
      Jpp_exp_e_ID : begin low_limit := Jpp_exp_e / 2; up_limit := Jpp_exp_e * 2; end;
      Jpp_exp_f_ID : begin low_limit := Jpp_exp_f / 2; up_limit := Jpp_exp_f * 2; end;
      Jpp_exp_b_ID : begin low_limit := Jpp_exp_b / 2; up_limit := Jpp_exp_b * 2; end;
      vers_ID :      begin low_limit := vers / 2; up_limit := vers * 2; end;
      AspRat_ID :    begin low_limit := AspRat / 2; up_limit := AspRat * 2; end;
    end;
  end;
  QTMFile.WriteFloat('DfDG_vs_actpar','low_limit',x_axis_low_limit);
  QTMFile.WriteFloat('DfDG_vs_actpar','up_limit',x_axis_up_limit);
  low_limit_Show.Text := FloatToStrF(low_limit,fffixed,8,3);
  up_limit_Show.Text := FloatToStrF(up_limit,fffixed,8,3);
  B_Changed := True;
  Simulate; Update_Graphs;
end;

procedure TDfDG_vs_actpar.low_limit_ShowKeyPress(Sender: TObject; var Key: Char);
begin
  with Sender as TEdit do begin
    Color := cl3dLight;
    if GetKeyState(VK_DECIMAL) < 0 then Key := DecimalSeparator;
    if key <> #13 then Exit;
    if key = #27 then begin Color := clwindow; Exit; end;
  end;
  low_limit := M.FloatFromEdit(Sender as TEdit, low_limit);
  QTMFile.WriteFloat('DfDG_vs_actpar','low_limit',low_limit); B_Changed := True;
  (Sender as TEdit).Text := FloatToStrf(low_limit,fffixed,8,3);
  (Sender as TEdit).color := clwindow;
  Simulate; Update_Graphs;
  if key = #13 then key := #0;
end;

procedure TDfDG_vs_actpar.up_limit_ShowKeyPress(Sender: TObject; var Key: Char);
begin
  with Sender as TEdit do begin
    Color := cl3dLight;
    if GetKeyState(VK_DECIMAL) < 0 then Key := DecimalSeparator;
    if key <> #13 then Exit;
    if key = #27 then begin Color := clwindow; Exit; end;
  end;
  up_limit := M.FloatFromEdit(Sender as TEdit, up_limit);
  QTMFile.WriteFloat('DfDG_vs_actpar','up_limit',up_limit); B_Changed := True;
  (Sender as TEdit).Text := FloatToStrf(up_limit,fffixed,8,3);
  (Sender as TEdit).color := clwindow;
  Simulate; Update_Graphs;
  if key = #13 then key := #0;
end;

end.
