unit QTM_Fit;
interface
uses  Windows,Messages,SysUtils,Classes,Graphics,Controls,Forms,Dialogs,
  Menus,StdCtrls,ComCtrls,ExtCtrls,Pesgvcl,Pegrpapi,Math,QTM_Decl,QTM_Main;
type
  TFit = class(TObject)
    Procedure Simplex(var Errorcode: Byte);
    function func_Simplex(a: TDouble_NP; var Errorcode: Byte): Double;

    private 
      p              : TDouble_MPbyNP;
      y              : TDouble_MP;
      iter,n_dat_p   : integer;
      n_dat          : Integer;

      procedure fguess(guess : TDouble_Np; var p : TDouble_MPbyNP; var y : TDouble_MP; var Errorcode: Byte);
      procedure amoeba(var p: TDouble_MPbyNP; var y: TDouble_MP; ndim: integer; ftol: Double;  var nfunc: integer; var Errorcode: Byte);
end;

var Fit : TFit;

implementation
uses QTM_SLA,QTM_MIO,QTM_Complex,QTM_Perturbation,QTM_Multiple,QTM_Limits;
var simplex_chisqr : Double;

function TFit.func_simplex(a: TDouble_NP; var Errorcode: Byte): Double;
var i : integer;
    chisqr,dummy : Double;
begin
  SLA.a_to_Pars(a);
  chisqr := 0;
  n_dat := 0;
  i := 1;
  while (i <= nhm) and (Errorcode = 0) do begin
    n_r := n_arr[i];
    if B_fit_inc[i] then begin
      case i_model of
        j_model_AMF : SLA.dfstar_from_Z(df_fit[i],dG_fit[i],dummy,dummy,Errorcode);
        j_model_PE3 : Pert.dfstar_Pe3(df_fit[i],dG_fit[i],dummy,dummy,Errorcode);
        j_model_PE5 : Pert.dfstar_Pe5(df_fit[i],dG_fit[i],dummy,dummy,Errorcode);
      end;
      case i_fit_df_DG of
        1 : begin
          case i_wei_ninv of
            1 :                 chisqr := chisqr +  sqr(df[i] - df_fit[i]);
            2 : if n_r > 0 then chisqr := chisqr + (sqr(df[i] - df_fit[i])) / n_r
                else            chisqr := chisqr + (sqr(df[i] - df_fit[i]));
            3 : if n_r > 0 then chisqr := chisqr + (sqr(df[i] - df_fit[i])) / sqr(n_r)
                else            chisqr := chisqr + (sqr(df[i] - df_fit[i]));
          end;
          n_dat := n_dat + 1;
        end;
        2 : begin
          case i_wei_ninv of
            1 :                 chisqr := chisqr +  sqr(dG[i] - dG_fit[i]);
            2 : if n_r > 0 then chisqr := chisqr + (sqr(dG[i] - dG_fit[i])) / n_r
                else            chisqr := chisqr +  sqr(dG[i] - dG_fit[i]) ;
            3 : if n_r > 0 then chisqr := chisqr + (sqr(dG[i] - dG_fit[i])) / sqr(n_r)
                else            chisqr := chisqr +  sqr(dG[i] - dG_fit[i]) ;
          end;
          n_dat := n_dat + 1;
        end;
        3 : begin
          case i_wei_ninv of
            1 :                 chisqr := chisqr +  sqr(df[i] - df_fit[i]) + sqr(dG[i] - dG_fit[i]);
            2 : if n_r > 0 then chisqr := chisqr + (sqr(df[i] - df_fit[i]) + sqr(dG[i] - dG_fit[i])) / n_r
                else            chisqr := chisqr +  sqr(df[i] - df_fit[i]) + sqr(dG[i] - dG_fit[i]);
            3 : if n_r > 0 then chisqr := chisqr + (sqr(df[i] - df_fit[i]) + sqr(dG[i] - dG_fit[i])) / sqr(n_r)
                else            chisqr := chisqr +  sqr(df[i] - df_fit[i]) + sqr(dG[i] - dG_fit[i]);
          end;
          n_dat := n_dat + 2;
        end;
      end;
    end;
    inc(i);
  end; //while
  if err_df > 0 then chisqr := chisqr / sqr(err_df);
  if n_dat - n_fit_pars > 0 then chisqr := chisqr / (n_dat - n_fit_pars);
  simplex_chisqr := chisqr;
  result := chisqr;
end;

procedure TFit.fguess(guess : TDouble_Np; var p : TDouble_MPbyNP; var y : TDouble_MP; var Errorcode: Byte);
var s,a,b : TDouble_NP;
    ss0 : Double;
    ndim,i,j,k,l : integer;
begin
  ndim := n_fit_pars;
  ss0 := (sqrt(ndim + 1) - 1) / (sqrt(2) * ndim);
  for i := 1 to ndim do begin
    p[1,i] := guess[i];
    if B_Limits_Active then Limits.Check_for_Limits(fit_pointer[i],p[1,i]);
  end;
  for j := 1 to ndim do begin
    a[j] := p[1,j] * 0.1 * ss0;
    b[j] := p[1,j] * 0.1 * (ss0 + 1/sqrt(2));
  end;
  for l := 2 to ndim + 1 do begin
    for k := 1 to ndim do begin
      p[l,k]   := p[1,k]+a[k];
      p[l,l-1] := p[1,l-1] + b[l-1];
      if B_Limits_Active then begin
        Limits.Check_for_Limits(fit_pointer[k  ],p[l  ,k]);
        Limits.Check_for_Limits(fit_pointer[l-1],p[l,l-1]);
      end;  
    end;
  end;
  for i := 1 to ndim + 1 do begin
    for j := 1 to ndim do s[j] := p[i,j];
    y[i] := func_simplex(s, Errorcode)
   end;
end;

procedure TFit.amoeba(var p: TDouble_MPbyNP; var y: TDouble_MP; ndim: integer; ftol: Double;  var nfunc: integer; var Errorcode: Byte);
const nfuncmax = 5000; alpha = 1.0; beta = 0.5; gamma = 2.0;
LABEL 99;
var
   mpts,j,inhi,ilo,ihi,i     : integer;
   ytry,ysave,sum,rtol       : Double;
   psum                      : TDouble_NP;

function amotry_B(var p: TDouble_MPbyNP; var y: TDouble_MP;  var sum: TDouble_NP; ndim,ihi: integer;
                var nfunc: integer; fac: Double; var Errorcode: Byte): Double;
var j: integer;
    fac1,fac2,ytry: Double;
    ptry: TDouble_NP;
begin
  fac1 := (1.0 - fac) / ndim; fac2 := fac1 - fac;
  for j := 1 to ndim do begin
    ptry[j] := sum[j] * fac1 - p[ihi,j] * fac2;
    if B_Limits_Active then Limits.Check_for_Limits(fit_pointer[j],ptry[j]);
  end;
  ytry := func_simplex(ptry, Errorcode);
  nfunc := nfunc + 1;
  if ytry < y[ihi] then begin
    y[ihi] := ytry;
    for j := 1 to ndim do begin sum[j] := sum[j] + ptry[j] - p[ihi,j]; p[ihi,j] := ptry[j] end
  end;
  amotry_B := ytry;
end;

begin
  mpts := ndim+1;
  nfunc := 0;
  for j := 1 to ndim do begin
    sum := 0;
    for i := 1 to mpts do sum := sum + p[i,j];
    psum[j] := sum
  end;
  while (Errorcode = 0) do begin
    ilo := 1;
    if y[1] > y[2] then begin ihi := 1; inhi := 2 end
    else                begin ihi := 2; inhi := 1 end;
    for i := 1 to mpts do begin
      if y[i] < y[ilo] then ilo := i;
      if y[i] > y[ihi] then begin inhi := ihi; ihi := i end
      else if y[i] > y[inhi] then
        if i <> ihi then inhi := i
    end;
    if (abs(y[ihi]) + abs(y[ilo])) > 0  then
      rtol := 2.0 * abs(y[ihi] - y[ilo]) / (abs(y[ihi]) + abs(y[ilo]))
    else rtol := 0;
    if rtol < ftol then goto 99;
    if nfunc >= nfuncmax then begin
      //MessageDlg('Pause in AMOEBA - too many iterations',mtinformation,[mbOK],0); PS
      Errorcode := Errorcode or E_Fit_Failed;
      goto 99;
    end;
    ytry := amotry_B(p,y,psum,ndim,ihi,nfunc,-alpha, Errorcode);
    if ytry <= y[ilo] then begin
      ytry := amotry_B(p,y,psum,ndim,ihi,nfunc,gamma, Errorcode)
    end
    else begin
      if ytry >= y[inhi] then begin
        ysave := y[ihi];
        ytry := amotry_B(p,y,psum,ndim,ihi,nfunc,beta, Errorcode);
        if ytry >= ysave then begin
          for i := 1 to mpts do
            if i <> ilo then begin
              for j := 1 to ndim do begin
                psum[j] := 0.5 * (p[i,j] + p[ilo,j]);
                p[i,j] := psum[j]
              end;
              y[i] := func_Simplex(psum, Errorcode)
            end;
          nfunc := nfunc + ndim;
          for j := 1 to ndim do begin
            sum := 0.0;
            for i := 1 to mpts do sum := sum+p[i,j];
            psum[j] := sum
          end
        end
      end
    end;
  end; //while
  99:
end;

Procedure TFit.Simplex(var Errorcode: Byte);
var i,ndim : integer;
begin
  try
    if n_fit_pars > 6 then begin
      MessageDLG('Maximum number of fit parameters is 6' + #13 +
       'Reduce number of fit parameters by unchecking squares',mtinformation,[mbOK],0);
       Exit;
    end;
    nhar_fit := 0;
    for i := 1 to nhm do if B_fit_inc[i] then nhar_fit := nhar_fit + 1;
    if nhar_fit = 0 then begin
      MessageDlg('Must have at least one harmonic active for fitting' + #13 +
                 'Harmonic no 1 was chosen',mtinformation,[mbOK],0);
      B_fit_inc[1] := True;
      QTMFile.WriteBool('Main','B_fit_inc' + inttostr(1),B_fit_inc[1]); B_Changed := True;
      M.B_inc_clBox.Checked[0] := True;
    end;
    if not B_from_multiple then MIO.pars_to_a(guess)
    else for i := 1 to n_fit_pars do guess[i] := Multiple.guess_multiple[i];

    for i := 1 to n_fit_pars do if guess[i] = 0 then guess[i] := 0.1;
    ndim := n_fit_pars; ftol := 1e-12;
    if ndim = 0 then begin
      MessageDlg('Select Fit Parameters',mtInformation,[mbOK],0);
      Errorcode := Errorcode or E_Fit_Failed;
      exit;
    end;
    fguess(guess,p,y, Errorcode);      //find guess
    amoeba(p,y,ndim,ftol,iter, Errorcode);       //fit
    if (Errorcode = 0) then begin
      n_dat_p := 0;
      for i := 1 to nhm do begin
        if B_fit_inc[i] then begin
          case i_fit_df_DG of
            1,2 : n_dat_p := n_dat_p + 1;
            3   : n_dat_p := n_dat_p + 2;
          end;
        end;
      end;
      for i := 1 to n_fit_pars do a_fit[i] := p[1,i];  //Fitparameters to p
      if n_dat_p > ndim then chisqr := simplex_chisqr/(n_dat_p - ndim) else chisqr := simplex_chisqr;
      SLA.a_to_Pars(a_fit);
    end
    else begin
      for i := 1 to n_fit_pars do a_fit[i] := 0;
      chisqr := 0;
      M.Chisqr_Label.Visible := True;
      M.Chisqr_Label.Caption := 'Fit Failed';
      Pars := BU_pars;
    end;
  except
    on EOverflow do Errorcode := Errorcode or E_Overflow;
    else raise;
  end;
end;

end.

 // amotry is obsolete, does not check for limits
{function amotry(var p: TDouble_MPbyNP; var y: TDouble_MP;  var sum: TDouble_NP; ndim,ihi: integer;
                var nfunc: integer; fac: Double): Double;
var j: integer;
    fac1,fac2,ytry: Double;
    ptry: TDouble_NP;
begin
  fac1 := (1.0 - fac) / ndim; fac2 := fac1 - fac;
  for j := 1 to ndim do ptry[j] := sum[j] * fac1 - p[ihi,j] * fac2;
  ytry := func_simplex(ptry, Errorcode);
  nfunc := nfunc + 1;
  if ytry < y[ihi] then begin
    y[ihi] := ytry;
    for j := 1 to ndim do begin sum[j] := sum[j] + ptry[j] - p[ihi,j]; p[ihi,j] := ptry[j] end
  end;
  amotry := ytry;
end;
}
