unit QTM_MIO;
interface
uses Windows,Messages,SysUtils,Classes,Graphics,Controls,Forms,Dialogs,
  Pesgvcl,StdCtrls,ExtCtrls,QTM_Decl,PEgrpapi,CheckLst,Menus,Math,QTM_complex,Inifiles,Clipbrd, ZLib;

type TMIO = class(Tobject)
  procedure System_Read( i_par_Rec : Integer; InFile:  TIniFile);
  procedure System_Write(i_par_Rec : Integer; OutFile: TIniFile);
  procedure System_Read_part( i_par,i_par_Rec : Integer; InFile:  TIniFile);
  procedure System_Write_part(i_par,i_par_Rec : Integer; OutFile: TIniFile);
  function str_w_dec_p(in_str : String) : String;
  procedure Avoid_Zeros(i_par_Rec : Integer);
  procedure pars_to_a(var a : TDouble_NP);
  procedure pars_to_x(var x : Double);
  procedure Set_IO_Labels_Main;
  procedure Determine_Fit_Pars;
  procedure Count_Fit_Pars;

  function Graph_Labels(ID: Integer): string;
  procedure GraphToClpbd(Graph: PESGraph);
  Procedure ExportStrToClpb(OutStr: string);
  public
    i_test : integer;
  private
end;

var MIO : TMIO;

implementation
uses QTM_Main,QTM_Multiple,QTM_Settings,QTM_Limits;

const CombinedDatafile: Int64 = 5572168109079228227;  //= 'Comb_QTM' Identifier for Combined qtz Datafiles (first entry, 8 Bytes)
      CompressedDataFile: Int64 = 5572168190918487875; //'ComprQTM'

procedure TMIO.System_Read(i_par_rec : Integer; InFile: TIniFile);
var i_par : integer;
begin
  for i_par := 1 to i_par_max do System_Read_Part(i_par,i_par_rec, Infile);
  case i_par_Rec of
    j_par_Sample : Pars := GPars;
    j_par_Ref    : Ref_Pars := GPars;
    j_par_Limits_Min : Min_Pars := GPars;
    j_par_Limits_Max : Max_Pars := GPars;
  end;
  Avoid_Zeros(i_par_rec);
  if Min_Pars.rho_b = Max_Pars.rho_b then Limits.Defaults1Click(Self);
end;

procedure TMIO.System_Write(i_par_Rec : Integer; OutFile: TIniFile);
var i_par : integer;
begin
  for i_par := 1 to i_par_max do System_Write_Part(i_par,i_par_Rec, OutFile);
end;

procedure TMIO.System_Read_part(i_par, i_par_Rec : Integer; InFile: TIniFile);
var sections : string;
begin
  case i_par_Rec of
    j_par_Sample     : sections := 'Sample';
    j_par_Ref        : sections := 'Reference';
    j_par_Limits_Min : sections := 'LimitsMin';
    j_par_Limits_Max : sections := 'LimitsMax';
  end;
  case i_par of
    t_e_ID                 : begin
      GPars.t_e                 := InFile.ReadFloat(sections,IO_labels[i_par],0);
      if not B_Layer_1 then begin
        GPars.t_e := 0; InFile.WriteFloat(sections,IO_labels[i_par],GPars.t_e);
      end;
    end;
    t_f_ID                 :  begin
      GPars.t_f                 := InFile.ReadFloat(sections,IO_labels[i_par],0);
      if not B_Layer_2 then begin
        GPars.t_f := 0; InFile.WriteFloat(sections,IO_labels[i_par],GPars.t_f);
      end;
    end;

    rho_e_ID               :        GPars.rho_e               := InFile.ReadFloat(sections,IO_labels[i_par],1);
    rho_f_ID               :        GPars.rho_f               := InFile.ReadFloat(sections,IO_labels[i_par],1);
    rho_b_ID               : begin
      GPars.rho_b := InFile.ReadFloat(sections,IO_labels[i_par],1);
      if not B_Liquid_bulk then begin
        GPars.rho_b := 0; InFile.WriteFloat(sections,IO_labels[i_par],GPars.rho_b);
      end;
    end;
    Jp_e_ID                :        GPars.Jp_e                := InFile.ReadFloat(sections,IO_labels[i_par],1);
    Jp_f_ID                :        GPars.Jp_f                := InFile.ReadFloat(sections,IO_labels[i_par],1);
    Jp_b_ID                :        GPars.Jp_b                := InFile.ReadFloat(sections,IO_labels[i_par],0.89);

    Jp_exp_e_ID            :        GPars.Jp_exp_e            := InFile.ReadFloat(sections,IO_labels[i_par],-0.5);
    Jp_exp_f_ID            :        GPars.Jp_exp_f            := InFile.ReadFloat(sections,IO_labels[i_par],-0.5);
    Jp_exp_b_ID            :        GPars.Jp_exp_b            := InFile.ReadFloat(sections,IO_labels[i_par],0);

    Jpp_e_ID               :        GPars.Jpp_e               := InFile.ReadFloat(sections,IO_labels[i_par],1);
    Jpp_f_ID               :        GPars.Jpp_f               := InFile.ReadFloat(sections,IO_labels[i_par],1);
    Jpp_b_ID               :        GPars.Jpp_b               := InFile.ReadFloat(sections,IO_labels[i_par],0);

    Jpp_exp_e_ID           :        GPars.Jpp_exp_e           := InFile.ReadFloat(sections,IO_labels[i_par],-0.5);
    Jpp_exp_f_ID           :        GPars.Jpp_exp_f           := InFile.ReadFloat(sections,IO_labels[i_par],-0.5);
    Jpp_exp_b_ID           :        GPars.Jpp_exp_b           := InFile.ReadFloat(sections,IO_labels[i_par],0);

    i_ve_e_ID              :        GPars.i_ve_e              := InFile.ReadInteger(sections,IO_labels[i_par],1);
    i_ve_f_ID              :        GPars.i_ve_f              := InFile.ReadInteger(sections,IO_labels[i_par],1);
    i_ve_b_ID              :        GPars.i_ve_b              := InFile.ReadInteger(sections,IO_labels[i_par],3);

    vers_ID                :        GPars.vers                := InFile.ReadFloat(sections,IO_labels[i_par],0);
    AspRat_ID              :        GPars.AspRat              := InFile.ReadFloat(sections,IO_labels[i_par],1);
  end;
end;

procedure TMIO.System_Write_part(i_par, i_par_Rec : Integer; OutFile: TIniFile);
begin
  case i_par_Rec of
    j_par_Sample     : begin GPars :=     Pars; sections := 'Sample';    end;
    j_par_Ref        : begin GPars := Ref_Pars; sections := 'Reference'; end;
    j_par_Limits_Min : begin GPars := Min_Pars; sections := 'LimitsMin'; end;
    j_par_Limits_Max : begin GPars := Max_Pars; sections := 'LimitsMax'; end;
  end;
  case i_par of
    t_e_ID : OutFile.WriteFloat(sections,IO_labels[i_par],GPars.t_e);
    t_f_ID : OutFile.WriteFloat(sections,IO_labels[i_par],GPars.t_f);

    rho_e_ID : OutFile.WriteFloat(sections,IO_labels[i_par],GPars.rho_e);
    rho_f_ID : OutFile.WriteFloat(sections,IO_labels[i_par],GPars.rho_f);
    rho_b_ID : OutFile.WriteFloat(sections,IO_labels[i_par],GPars.rho_b);

    Jp_e_ID : OutFile.WriteFloat(sections,IO_labels[i_par],GPars.Jp_e);
    Jp_f_ID : OutFile.WriteFloat(sections,IO_labels[i_par],GPars.Jp_f);
    Jp_b_ID : OutFile.WriteFloat(sections,IO_labels[i_par],GPars.Jp_b);

    Jp_exp_e_ID : OutFile.WriteFloat(sections,IO_labels[i_par],GPars.Jp_exp_e);
    Jp_exp_f_ID : OutFile.WriteFloat(sections,IO_labels[i_par],GPars.Jp_exp_f);
    Jp_exp_b_ID : OutFile.WriteFloat(sections,IO_labels[i_par],GPars.Jp_exp_b);

    Jpp_e_ID : OutFile.WriteFloat(sections,IO_labels[i_par],GPars.Jpp_e);
    Jpp_f_ID : OutFile.WriteFloat(sections,IO_labels[i_par],GPars.Jpp_f);
    Jpp_b_ID : OutFile.WriteFloat(sections,IO_labels[i_par],GPars.Jpp_b);

    Jpp_exp_e_ID : OutFile.WriteFloat(sections,IO_labels[i_par],GPars.Jpp_exp_e);
    Jpp_exp_f_ID : OutFile.WriteFloat(sections,IO_labels[i_par],GPars.Jpp_exp_f);
    Jpp_exp_b_ID : OutFile.WriteFloat(sections,IO_labels[i_par],GPars.Jpp_exp_b);

    i_ve_e_ID : OutFile.WriteInteger(sections,IO_labels[i_par],GPars.i_ve_e);
    i_ve_f_ID : OutFile.WriteInteger(sections,IO_labels[i_par],GPars.i_ve_f);
    i_ve_b_ID : OutFile.WriteInteger(sections,IO_labels[i_par],GPars.i_ve_b);

    vers_ID : OutFile.WriteFloat(sections,IO_labels[i_par],GPars.vers);
    AspRat_ID : OutFile.WriteFloat(sections,IO_labels[i_par],GPars.AspRat);
  end;
  B_Changed := True;
end;

function TMIO.str_w_dec_p(in_str : String) : String;
var i : integer;
    test : string;
begin
  i_test := length(in_str);
  test := '';
  for i := 1 to length(in_str) do begin
    if in_str[i] <> ',' then test := test + in_str[i]
    else test := test + '.';
  end;
  str_w_dec_p := test;
end;

procedure TMIO.Pars_to_a(var a : TDouble_NP);  //bergabe der Fitparameter an a (Fitarray)
var i : integer;
begin
  for i := 1 to n_fit_pars do begin
    case fit_pointer[i] of
      t_e_ID         : begin
        a[i] := Pars.t_e;
        if a[i] = Max_Pars.t_e then a[i] := Min_Pars.t_e + 0.99 * (Max_Pars.t_e - Min_Pars.t_e);
        if a[i] = Min_Pars.t_e then a[i] := Min_Pars.t_e + 0.01 * (Max_Pars.t_e - Min_Pars.t_e);
      end;
      t_f_ID         : begin
        a[i] := Pars.t_f;
        if a[i] = Max_Pars.t_f then a[i] := Min_Pars.t_f + 0.99 * (Max_Pars.t_f - Min_Pars.t_f);
        if a[i] = Min_Pars.t_f then a[i] := Min_Pars.t_f + 0.01 * (Max_Pars.t_f - Min_Pars.t_f);
      end;
      rho_e_ID       : begin
        a[i] := Pars.rho_e;
        if a[i] = Max_Pars.rho_e then a[i] := Min_Pars.rho_e + 0.99 * (Max_Pars.rho_e - Min_Pars.rho_e);
        if a[i] = Min_Pars.rho_e then a[i] := Min_Pars.rho_e + 0.01 * (Max_Pars.rho_e - Min_Pars.rho_e);
      end;
      rho_f_ID       : begin
        a[i] := Pars.rho_f;
        if a[i] = Max_Pars.rho_f then a[i] := Min_Pars.rho_f + 0.99 * (Max_Pars.rho_f - Min_Pars.rho_f);
        if a[i] = Min_Pars.rho_f then a[i] := Min_Pars.rho_f + 0.01 * (Max_Pars.rho_f - Min_Pars.rho_f);
      end;
      rho_b_ID       : begin
        a[i] := Pars.rho_b;
        if a[i] = Max_Pars.rho_b then a[i] := Min_Pars.rho_b + 0.99 * (Max_Pars.rho_b - Min_Pars.rho_b);
        if a[i] = Min_Pars.rho_b then a[i] := Min_Pars.rho_b + 0.01 * (Max_Pars.rho_b - Min_Pars.rho_b);
      end;
      Jp_e_ID        : begin
        a[i] := Pars.Jp_e;
        if a[i] = Max_Pars.Jp_e then a[i] := Min_Pars.Jp_e + 0.99 * (Max_Pars.Jp_e - Min_Pars.Jp_e);
        if a[i] = Min_Pars.Jp_e then a[i] := Min_Pars.Jp_e + 0.01 * (Max_Pars.Jp_e - Min_Pars.Jp_e);
      end;
      Jp_f_ID        : begin
        a[i] := Pars.Jp_f;
        if a[i] = Max_Pars.Jp_f then a[i] := Min_Pars.Jp_f + 0.99 * (Max_Pars.Jp_f - Min_Pars.Jp_f);
        if a[i] = Min_Pars.Jp_f then a[i] := Min_Pars.Jp_f + 0.01 * (Max_Pars.Jp_f - Min_Pars.Jp_f);
      end;
      Jp_b_ID        : begin
        a[i] := Pars.Jp_b;
        if a[i] = Max_Pars.Jp_b then a[i] := Min_Pars.Jp_b + 0.99 * (Max_Pars.Jp_b - Min_Pars.Jp_b);
        if a[i] = Min_Pars.Jp_b then a[i] := Min_Pars.Jp_b + 0.01 * (Max_Pars.Jp_b - Min_Pars.Jp_b);
      end;
      Jp_exp_e_ID    : begin
        a[i] := Pars.Jp_exp_e;
        if a[i] = Max_Pars.Jp_exp_e then a[i] := Min_Pars.Jp_exp_e + 0.99 * (Max_Pars.Jp_exp_e - Min_Pars.Jp_exp_e);
        if a[i] = Min_Pars.Jp_exp_e then a[i] := Min_Pars.Jp_exp_e + 0.01 * (Max_Pars.Jp_exp_e - Min_Pars.Jp_exp_e);
      end;
      Jp_exp_f_ID    : begin
        a[i] := Pars.Jp_exp_f;
        if a[i] = Max_Pars.Jp_exp_f then a[i] := Min_Pars.Jp_exp_f + 0.99 * (Max_Pars.Jp_exp_f - Min_Pars.Jp_exp_f);
        if a[i] = Min_Pars.Jp_exp_f then a[i] := Min_Pars.Jp_exp_f + 0.01 * (Max_Pars.Jp_exp_f - Min_Pars.Jp_exp_f);
      end;
      Jp_exp_b_ID    : begin
        a[i] := Pars.Jp_exp_b;
        if a[i] = Max_Pars.Jp_exp_b then a[i] := Min_Pars.Jp_exp_b + 0.99 * (Max_Pars.Jp_exp_b - Min_Pars.Jp_exp_b);
        if a[i] = Min_Pars.Jp_exp_b then a[i] := Min_Pars.Jp_exp_b + 0.01 * (Max_Pars.Jp_exp_b - Min_Pars.Jp_exp_b);
      end;
      Jpp_e_ID       : begin
        a[i] := Pars.Jpp_e;
        if a[i] = Max_Pars.Jpp_e then a[i] := Min_Pars.Jpp_e + 0.99 * (Max_Pars.Jpp_e - Min_Pars.Jpp_e);
        if a[i] = Min_Pars.Jpp_e then a[i] := Min_Pars.Jpp_e + 0.01 * (Max_Pars.Jpp_e - Min_Pars.Jpp_e);
      end;
      Jpp_f_ID       : begin
        a[i] := Pars.Jpp_f;
        if a[i] = Max_Pars.Jpp_f then a[i] := Min_Pars.Jpp_f + 0.99 * (Max_Pars.Jpp_f - Min_Pars.Jpp_f);
        if a[i] = Min_Pars.Jpp_f then a[i] := Min_Pars.Jpp_f + 0.01 * (Max_Pars.Jpp_f - Min_Pars.Jpp_f);
      end;
      Jpp_b_ID       : begin
        a[i] := Pars.Jpp_b;
        if a[i] = Max_Pars.Jpp_b then a[i] := Min_Pars.Jpp_b + 0.99 * (Max_Pars.Jpp_b - Min_Pars.Jpp_b);
        if a[i] = Min_Pars.Jpp_b then a[i] := Min_Pars.Jpp_b + 0.01 * (Max_Pars.Jpp_b - Min_Pars.Jpp_b);
      end;
      Jpp_exp_e_ID   : begin
        a[i] := Pars.Jpp_exp_e;
        if a[i] = Max_Pars.Jpp_exp_e then a[i] := Min_Pars.Jpp_exp_e + 0.99 * (Max_Pars.Jpp_exp_e - Min_Pars.Jpp_exp_e);
        if a[i] = Min_Pars.Jpp_exp_e then a[i] := Min_Pars.Jpp_exp_e + 0.01 * (Max_Pars.Jpp_exp_e - Min_Pars.Jpp_exp_e);
      end;
      Jpp_exp_f_ID   : begin
        a[i] := Pars.Jpp_exp_f;
        if a[i] = Max_Pars.Jpp_exp_f then a[i] := Min_Pars.Jpp_exp_f + 0.99 * (Max_Pars.Jpp_exp_f - Min_Pars.Jpp_exp_f);
        if a[i] = Min_Pars.Jpp_exp_f then a[i] := Min_Pars.Jpp_exp_f + 0.01 * (Max_Pars.Jpp_exp_f - Min_Pars.Jpp_exp_f);
      end;
      Jpp_exp_b_ID   : begin
        a[i] := Pars.Jpp_exp_b;
        if a[i] = Max_Pars.Jpp_exp_b then a[i] := Min_Pars.Jpp_exp_b + 0.99 * (Max_Pars.Jpp_exp_b - Min_Pars.Jpp_exp_b);
        if a[i] = Min_Pars.Jpp_exp_b then a[i] := Min_Pars.Jpp_exp_b + 0.01 * (Max_Pars.Jpp_exp_b - Min_Pars.Jpp_exp_b);
      end;
      vers_ID        : begin
        a[i] := Pars.vers;
        if a[i] = Max_Pars.vers then a[i] := Min_Pars.vers + 0.99 * (Max_Pars.vers - Min_Pars.vers);
        if a[i] = Min_Pars.vers then a[i] := Min_Pars.vers + 0.01 * (Max_Pars.vers - Min_Pars.vers);
      end;
      AspRat_ID      : begin
        a[i] := Pars.AspRat;
        if a[i] = Max_Pars.AspRat then a[i] := Min_Pars.AspRat + 0.99 * (Max_Pars.AspRat - Min_Pars.AspRat);
        if a[i] = Min_Pars.AspRat then a[i] := Min_Pars.AspRat + 0.01 * (Max_Pars.AspRat - Min_Pars.AspRat);
      end;
    end;
  end;
end;

procedure TMIO.Pars_to_x(var x : Double);  //bergabe der Fitparameter an x (dfdg_vs_actpor
begin
  case i_rBtn of
    t_e_ID         : x := Pars.t_e;
    t_f_ID         : x := Pars.t_f;
    rho_e_ID       : x := Pars.rho_e;
    rho_f_ID       : x := Pars.rho_f;
    rho_b_ID       : x := Pars.rho_b;
    Jp_e_ID        : x := Pars.Jp_e;
    Jp_f_ID        : x := Pars.Jp_f;
    Jp_b_ID        : x := Pars.Jp_b;
    Jp_exp_e_ID    : x := Pars.Jp_exp_e;
    Jp_exp_f_ID    : x := Pars.Jp_exp_f;
    Jp_exp_b_ID    : x := Pars.Jp_exp_b;
    Jpp_e_ID       : x := Pars.Jpp_e;
    Jpp_f_ID       : x := Pars.Jpp_f;
    Jpp_b_ID       : x := Pars.Jpp_b;
    Jpp_exp_e_ID   : x := Pars.Jpp_exp_e;
    Jpp_exp_f_ID   : x := Pars.Jpp_exp_f;
    Jpp_exp_b_ID   : x := Pars.Jpp_exp_b;
    vers_ID        : x := Pars.vers;
    AspRat_ID      : x := Pars.AspRat;
  end;
end;


procedure TMIO.Count_Fit_Pars;
var i_count : integer;
begin
  i_count := 0;
  if B_t_e_C then inc(i_count);
  if B_t_f_C then inc(i_count);
  if B_rho_e_C then inc(i_count);
  if B_rho_f_C then inc(i_count);
  if B_rho_b_C then inc(i_count);
  if B_Jp_e_C then inc(i_count);
  if B_Jp_f_C then inc(i_count);
  if B_Jp_b_C then inc(i_count);
  if B_Jp_exp_e_C then inc(i_count);
  if B_Jp_exp_f_C then inc(i_count);
  if B_Jp_exp_b_C then inc(i_count);
  if B_Jpp_e_C then inc(i_count);
  if B_Jpp_f_C then inc(i_count);
  if B_Jpp_b_C then inc(i_count);
  if B_Jpp_exp_e_C then inc(i_count);
  if B_Jpp_exp_f_C then inc(i_count);
  if B_Jpp_exp_b_C then inc(i_count);
  if B_vers_C then inc(i_count);
  if B_AspRat_C then inc(i_count);
  n_fit_pars := i_count;
end;

procedure TMIO.Determine_Fit_Pars;
var i,i_count : Integer;
begin
  Count_Fit_Pars;
  if n_fit_pars > n_fit_pars_max then begin
    Messagedlg('Reduce number of fit parameters to <= 6',mtinformation,[mbOK],0);
    Exit;
  end;
  for i := 1 to n_fit_pars_max do fit_pointer[i] := 0;
  i_count := 0;
  if B_t_e_C then begin i_count := i_count + 1; fit_pointer[i_count] :=          t_e_ID; end;
  if B_t_f_C then begin i_count := i_count + 1; fit_pointer[i_count] :=          t_f_ID; end;
  if B_rho_e_C then begin i_count := i_count + 1; fit_pointer[i_count] :=        rho_e_ID; end;
  if B_rho_f_C then begin i_count := i_count + 1; fit_pointer[i_count] :=        rho_f_ID; end;
  if B_rho_b_C then begin i_count := i_count + 1; fit_pointer[i_count] :=        rho_b_ID; end;
  if B_Jp_e_C then begin i_count := i_count + 1; fit_pointer[i_count] :=         Jp_e_ID; end;
  if B_Jp_f_C then begin i_count := i_count + 1; fit_pointer[i_count] :=         Jp_f_ID; end;
  if B_Jp_b_C then begin i_count := i_count + 1; fit_pointer[i_count] :=         Jp_b_ID; end;
  if B_Jp_exp_e_C then begin i_count := i_count + 1; fit_pointer[i_count] :=     Jp_exp_e_ID; end;
  if B_Jp_exp_f_C then begin i_count := i_count + 1; fit_pointer[i_count] :=     Jp_exp_f_ID; end;
  if B_Jp_exp_b_C then begin i_count := i_count + 1; fit_pointer[i_count] :=     Jp_exp_b_ID; end;
  if B_Jpp_e_C then begin i_count := i_count + 1; fit_pointer[i_count] :=        Jpp_e_ID; end;
  if B_Jpp_f_C then begin i_count := i_count + 1; fit_pointer[i_count] :=        Jpp_f_ID; end;
  if B_Jpp_b_C then begin i_count := i_count + 1; fit_pointer[i_count] :=        Jpp_b_ID; end;
  if B_Jpp_exp_e_C then begin i_count := i_count + 1; fit_pointer[i_count] :=    Jpp_exp_e_ID; end;
  if B_Jpp_exp_f_C then begin i_count := i_count + 1; fit_pointer[i_count] :=    Jpp_exp_f_ID; end;
  if B_Jpp_exp_b_C then begin i_count := i_count + 1; fit_pointer[i_count] :=    Jpp_exp_b_ID; end;
  if B_vers_C then begin i_count := i_count + 1; fit_pointer[i_count] :=       vers_ID; end;
  if B_AspRat_C then begin i_count := i_count + 1; fit_pointer[i_count] :=     AspRat_ID; end;
  M.n_fit_pars_label.Caption := 'Fit par''s : ' + IntToStr(n_fit_pars);
  if Assigned(Multiple) then begin
    if (Multiple.Showing and not(Multiple.WindowState = wsMinimized)) then Multiple.Update_Values;
  end;
end;

procedure TMIO.Avoid_Zeros(i_par_Rec : Integer);
var B_zero_found : Boolean;
    message_str : String;
begin
  case i_par_Rec of
    j_par_Sample : GPars := Pars;
    j_par_Ref    : GPars := Ref_Pars;
    j_par_Limits_Min,j_par_Limits_Max : Exit;
  end;
  B_zero_found := False;
  message_str := '';
  if (GPars.t_e <> 0) then begin
    case GPars.i_ve_e of
      1 : if (sqr(GPars.Jp_e) +  sqr(GPars.Jpp_e) = 0) then begin message_str := 'Vanishing compliance encountered for layer 1'; B_zero_found := True; end;
      2 : if (sqr(GPars.Jp_e) +  sqr(GPars.Jpp_e) = 0) then begin message_str := 'Vanishing modulus encountered for layer 1';    B_zero_found := True; end;
      3 : if (sqr(GPars.Jp_e) +  sqr(GPars.Jpp_e) = 0) then begin message_str := 'Vanishing viscosity encountered for layer 1';  B_zero_found := True; end;
      4 : if (sqr(GPars.Jp_e) +  sqr(GPars.Jpp_e) = 0) then begin message_str := 'Vanishing modulus encountered for layer 1';    B_zero_found := True; end;
      5 : if (sqr(GPars.Jp_e) = 0) then begin message_str := 'Vanishing compliance encountered for layer 1';                     B_zero_found := True; end;
      6 : if (sqr(GPars.Jp_e) = 0) then begin message_str := 'Vanishing modulus encountered for layer 1';                        B_zero_found := True; end;
      7 : if (sqr(GPars.Jp_e) = 0) then begin message_str := 'Vanishing viscosity encountered for layer 1';                      B_zero_found := True; end;
    end;
    if B_zero_found then begin
      message_str := message_str + #13 + 'Was replaced,check';
      MessageDlg(message_str,mtinformation,[mbOK],0);
      case i_par_Rec of
        1 : Pars.Jp_e := 1;
        2 : Ref_Pars.Jp_e := 1;
      end;
      System_Write_Part(Jp_e_ID,i_par_Rec,QTMFile);
    end;
  end;

  B_zero_found := False; message_str := '';
  if (GPars.t_f <> 0) then begin
    case GPars.i_ve_f of
      1 : if (sqr(GPars.Jp_f) +  sqr(GPars.Jpp_f) = 0) then begin message_str := 'Vanishing compliance encountered for layer 2'; B_zero_found := True; end;
      2 : if (sqr(GPars.Jp_f) +  sqr(GPars.Jpp_f) = 0) then begin message_str := 'Vanishing modulus encountered for layer 2';    B_zero_found := True; end;
      3 : if (sqr(GPars.Jp_f) +  sqr(GPars.Jpp_f) = 0) then begin message_str := 'Vanishing viscosity encountered for layer 2';  B_zero_found := True; end;
      4 : if (sqr(GPars.Jp_f) +  sqr(GPars.Jpp_f) = 0) then begin message_str := 'Vanishing modulus encountered for layer 2';    B_zero_found := True; end;
      5 : if (sqr(GPars.Jp_f) = 0) then begin message_str := 'Vanishing compliance encountered for layer 2';                     B_zero_found := True; end;
      6 : if (sqr(GPars.Jp_f) = 0) then begin message_str := 'Vanishing modulus encountered for layer 2';                        B_zero_found := True; end;
      7 : if (sqr(GPars.Jp_f) = 0) then begin message_str := 'Vanishing viscosity encountered for layer 2';                      B_zero_found := True; end;
    end;
    if B_zero_found then begin
      message_str := message_str + #13 + 'Was replaced,check';
      MessageDlg(message_str,mtinformation,[mbOK],0);
      case i_par_Rec of
        1 : Pars.Jp_f := 1;
        2 : Ref_Pars.Jp_f := 1;
      end;
      System_Write_Part(Jp_f_ID,i_par_Rec,QTMFile);
    end;
  end;

  B_zero_found := False; message_str := '';
  if (GPars.rho_b <> 0) then begin
    case GPars.i_ve_b of
      1 : if (sqr(GPars.Jp_b) +  sqr(GPars.Jpp_b) = 0) then begin message_str := 'Vanishing compliance encountered for bulk medium'; B_zero_found := True; end;
      2 : if (sqr(GPars.Jp_b) +  sqr(GPars.Jpp_b) = 0) then begin message_str := 'Vanishing modulus encountered for bulk medium';    B_zero_found := True; end;
      3 : if (sqr(GPars.Jp_b) +  sqr(GPars.Jpp_b) = 0) then begin message_str := 'Vanishing viscosity encountered for bulk medium';  B_zero_found := True; end;
      4 : if (sqr(GPars.Jp_b) +  sqr(GPars.Jpp_b) = 0) then begin message_str := 'Vanishing modulus encountered for bulk medium';    B_zero_found := True; end;
      5 : if (sqr(GPars.Jp_b) = 0) then begin message_str := 'Vanishing compliance encountered for bulk medium';                     B_zero_found := True; end;
      6 : if (sqr(GPars.Jp_b) = 0) then begin message_str := 'Vanishing modulus encountered for bulk medium';                        B_zero_found := True; end;
      7 : if (sqr(GPars.Jp_b) = 0) then begin message_str := 'Vanishing viscosity encountered for bulk medium';                      B_zero_found := True; end;
    end;
    if B_zero_found then begin
      message_str := message_str + #13 + 'Was replaced,check';
      MessageDlg(message_str,mtinformation,[mbOK],0);
      case i_par_Rec of
        1 : Pars.Jp_b := 1;
        2 : Ref_Pars.Jp_b := 1;
      end;
      System_Write_Part(Jp_b_ID,i_par_Rec,QTMFile);
    end;
  end;

  if (GPars.t_e <> 0) and (GPars.rho_e = 0) then begin
    MessageDlg('Zero density encountered for layer 1' + #13 +
    'Was replaced,check ',mtinformation,[mbOK],0);
    case i_par_Rec of
      1 : Pars.rho_e := 1;
      2 : Ref_Pars.rho_e := 1;
    end;
    System_Write_Part(rho_e_ID,i_par_Rec,QTMFile);
  end;
  if (GPars.t_f <> 0) and (GPars.rho_f = 0) then begin
    MessageDlg('Zero density encountered for layer 2' + #13 +
    'Was replaced,check ',mtinformation,[mbOK],0);
    case i_par_Rec of
      1 : Pars.rho_f := 1;
      2 : Ref_Pars.rho_f := 1;
    end;
    System_Write_Part(rho_f_ID,i_par_Rec,QTMFile);
  end;

  if f_fund = 0 then begin
    MessageDlg('Zero f_fund encountered ' + #13 + 'Was replaced,check ',mtinformation,[mbOK],0);
    f_fund := 5e6;
    QTMFile.WriteFloat('Settings','f_fund',f_fund); B_Changed := True;
    M.Update_Values;
  end;

  if GPars.i_ve_e = 0  then begin
    case i_par_Rec of
      1 : Pars.i_ve_e  := 2;
      2 : Ref_Pars.i_ve_e  := 2
    end;
    System_Write_Part(i_ve_e_ID,i_par_Rec,QTMFile);
   end;
  if GPars.i_ve_f = 0  then begin
    case i_par_Rec of
      1 : Pars.i_ve_f  := 2;
      2 : Ref_Pars.i_ve_f  := 2
    end;
    System_Write_Part(i_ve_f_ID,i_par_Rec,QTMFile);
  end;
  if GPars.i_ve_b = 0  then begin
    case i_par_Rec of
      1 : Pars.i_ve_b  := 2;
      2 : Ref_Pars.i_ve_b  := 2
    end;
    System_Write_Part(i_ve_b_ID,i_par_Rec,QTMFile);
  end;

  if B_zero_found then begin
    case i_par_Rec of
      1 : begin M.Update_Values;        Pars := GPars;     end;
      2 : begin Settings.Update_Values; Ref_Pars := GPars; end;
    end;
  end;
end;

procedure TMIO.Set_IO_Labels_Main;
begin
  IO_labels[t_e_ID] := 'Thickness Layer 1 [nm]';
  IO_labels[t_f_ID] := 'Thickness Layer 2 [nm]';
  IO_labels[rho_e_ID] := 'Density Layer 1 [g/cm^3]';
  IO_labels[rho_f_ID] := 'Density Layer 2 [g/cm^3]';
  IO_labels[rho_b_ID] := 'Density Bulk [g/cm^3]';

  IO_labels[Jp_e_ID] := 'VE Par 1 Layer 1';
  IO_labels[Jp_f_ID] := 'VE Par 1 Layer 2';
  IO_labels[Jp_b_ID] := 'VE Par 1 Bulk';

  IO_labels[Jp_exp_e_ID] := 'Power Law Expon. beta_p Layer 1';
  IO_labels[Jp_exp_f_ID] := 'Power Law Expon. beta_p Layer 2';
  IO_labels[Jp_exp_b_ID] := 'Power Law Expon. beta_p Bulk';

  IO_labels[Jpp_e_ID] := 'VE par 2 Layer 1';
  IO_labels[Jpp_f_ID] := 'VE par 2 Layer 2';
  IO_labels[Jpp_b_ID] := 'VE par 2 Bulk';

  IO_labels[Jpp_exp_e_ID] := 'Power Law Expon. beta_pp Layer 1';
  IO_labels[Jpp_exp_f_ID] := 'Power Law Expon. beta_pp Layer 2';
  IO_labels[Jpp_exp_b_ID] := 'Power Law Expon. beta_pp Bulk';

  IO_labels[i_ve_e_ID] := 'Choice of VE Pars, Layer 1';
  IO_labels[i_ve_f_ID] := 'Choice of VE Pars, Layer 2';
  IO_labels[i_ve_b_ID] := 'Choice of VE Pars, Bulk';
  IO_labels[vers_ID]   := 'Vertical Scale of Roughness [nm]';
  IO_labels[AspRat_ID] := 'Aspect Ratio';
end;

function TMIO.Graph_Labels(ID: Integer): string;
begin
  case ID of
    t_e_ID:   result := 'Thickness Layer 1 [nm]';
    t_f_ID:   result := 'Thickness Layer 2 [nm]';
    rho_e_ID: result := 'Density Layer 1 [g/cm^3]';
    rho_f_ID: result := 'Density Layer 2 [g/cm^3]';
    rho_b_ID: result := 'Density Bulk Medium [g/cm^3]';

    Jp_e_ID: begin
      case Pars.i_ve_e of
        1 :result := 'J'' Layer 1 [MPa^-1]';
        2 :result := 'G'' Layer 1 [MPa]';
        3 :result := 'eta'' Layer 1 [mPa s]';
        4 :result := 'G'' Layer 1 [MPa]';
        5 :result := '|J| Layer 1 [MPa^-1]';
        6 :result := '|G| Layer 1 [MPa]';
        7 :result := '|eta| Layer 1 [mPa s]';
      end;
    end;

    Jpp_e_ID: begin
      case Pars.i_ve_e of
        1 : result := 'J'''' Layer 1 [MPa^-1]';
        2 : result := 'G'''' Layer 1 [MPa]';
        3 : result := 'eta'''' Layer 1 [mPa s]';
        4 : result := 'eta'' Layer 1 [mPa s]';
        5 : result := 'Loss Tangent Layer 1';
        6 : result := 'Loss Tangent Layer 1';
        7 : result := 'Inv. Loss Tangent Layer 1';
      end;
    end;

    Jp_f_ID: begin
      if Pars.i_ve_f = 0 then Pars.i_ve_f := 2;
      case Pars.i_ve_f of
        1 : result := 'J'' Layer 2 [MPa^-1]';
        2 : result := 'G'' Layer 2 [MPa]';
        3 : result := 'eta'' Layer 2 [mPa s]';
        4 : result := 'G'' Layer 2 [MPa]';
        5 : result := '|J| Layer 2 [MPa^-1]';
        6 : result := '|G| Layer 2 [MPa]';
        7 : result := '|eta| Layer 2 [mPa s]';
      end;
    end;

    Jpp_f_ID: begin
      if Pars.i_ve_f = 0 then Pars.i_ve_f := 2;
      case Pars.i_ve_f of
        1 : result := 'J'''' Layer 2 [MPa^-1]';
        2 : result := 'G'''' Layer 2 [MPa]';
        3 : result := 'eta'''' Layer 2 [mPa s]';
        4 : result := 'eta'' Layer 2 [mPa s]';
        5 : result := 'Loss Tangent Layer 2';
        6 : result := 'Loss Tangent Layer 2';
        7 : result := 'Inv. Loss Tangent Layer 2';
      end;
    end;

    Jp_b_ID: begin
      if Pars.i_ve_b = 0 then Pars.i_ve_b := 3;
      case Pars.i_ve_b of
        1 :  result := 'J'' Bulk [MPa^-1]';
        2 :  result := 'G'' Bulk [MPa]';
        3 :  result := 'eta'' Bulk [mPa s]';
        4 :  result := 'G'' Bulk [MPa]';
        5 :  result := '|J| Bulk [MPa^-1]';
        6 :  result := '|G| Bulk [MPa]';
        7 :  result := '|eta| Bulk [mPa s]';
      end;
    end;

    Jpp_b_ID: begin
      if Pars.i_ve_b = 0 then Pars.i_ve_b := 3;
      case Pars.i_ve_b of
        1 : result := 'J'''' Bulk [MPa^-1]';
        2 : result := 'G'''' Bulk [MPa]';
        3 : result := 'eta'''' Bulk [mPa s]';
        4 : result := 'eta'' Bulk [mPa s]';
        5 : result := 'Loss Tangent Bulk';
        6 : result := 'Loss Tangent Bulk';
        7 : result := 'Inv. Loss Tangent Bulk';
      end;
    end;
    Jp_exp_e_ID:            result  := 'Power Law Expon. beta '' Layer 1';
    Jp_exp_f_ID:            result  := 'Power Law Expon. beta '' Layer 2';
    Jp_exp_b_ID:            result  := 'Power Law Expon. beta '' Bulk';
    Jpp_exp_e_ID:           result  := 'Power Law Expon. beta '''' Layer 1';
    Jpp_exp_f_ID:           result  := 'Power Law Expon. beta '''' Layer 2';
    Jpp_exp_b_ID:           result  := 'Power Law Expon. beta '''' Bulk';
    vers_ID:                result  := 'Vertical Scale of Roughness  [nm]';
    AspRat_ID:              result  := 'Aspect Ratio';
  end;
end;

procedure TMIO.GraphToClpbd(Graph: PESGraph);
begin
  with Graph do begin
    PEnarg1 := Width*100;
    PEnarg2 := Height*100;
    PEactions := sgMetaToClip;
  end;
end;

Procedure TMIO.ExportStrToClpb(OutStr: string);
var SysClip: TClipBoard;
begin
  Screen.Cursor := crHourglass;
  SysClip := TClipBoard.Create;
  try
    SysClip.AsText := OutStr;
  finally
    SysClip.Free;
    Screen.Cursor := crDefault;
  end
end;

end.



