unit QTM_Perturbation;
interface
uses Dialogs,Messages,Controls,StdCtrls,SysUtils,
  QTM_Decl,QTM_Complex,QTM_SLA,QTM_Main,Math;

type TPert = Class(Tobject)
  procedure Assign_Pert(GPars : TPars; var Errorcode:Byte);
  procedure dfdf_Pe3(GPars : TPars; var dfdf,dfdf_Rou : TComplex; var Errorcode:Byte);
  procedure dfstar_Pe3(var df_Pert,dG_Pert,df_Rou,dG_Rou : Double; var Errorcode:Byte);
  procedure dfdf_Pe5(GPars : TPars; var dfdf,dfdf_Rou : TComplex; var Errorcode:Byte);
  procedure dfstar_Pe5(var df_Pert,dG_Pert,df_Rou,dG_Rou : Double; var Errorcode:Byte);

  private
    mu_e,J_e,Z_e_sqr,ze_e,mu_f,J_f,Z_f_sqr,ze_f,J_b,Z_b,xi_l,xi_l_no_Rou,
    Pi_c,n_c : TComplex;
    mq,om : Double;
end;

var Pert : TPert;

implementation

procedure TPert.Assign_Pert(GPars : TPars; var Errorcode:Byte);
begin
  one_c := Complex(1,0); im_c := Complex(0,1); zero_c := Complex(0.0,0.0);
  Pi_c := complex(Pi,0);
  n_c := Complex(n_r,0);
  mq := Zq / (2 * f_fund);
  om := 2 * pi * n_r * f_fund;

  mu_e := Complex(GPars.t_e * 1e-9 * GPars.rho_e * 1e3 / mq,0);
  if abs_c(mu_e) > 0 then begin
    J_e := SLA.Jc(1,n_r,GPars);
    if abs_c(J_e) > 0 then begin
      Z_e_sqr := div_c(complex(GPars.rho_e * 1e3,0),J_e,Errorcode);
      ze_e := plus_c(div_c(complex(sqr(Zq),0),Z_e_sqr,Errorcode),complex(-1,0));
    end else begin
      ze_e := complex(-1,0);
    end;
  end else begin
    ze_e := complex(-1,0);
  end;

  mu_f := Complex(GPars.t_f * 1e-9 * GPars.rho_f * 1e3 / mq,0);
  if abs_c(mu_f) > 0 then begin
    J_f := SLA.Jc(2,n_r,GPars);
    if abs_c(J_f) > 0 then begin
      Z_f_sqr := div_c(complex(GPars.rho_f * 1e3,0),J_f,Errorcode);
      ze_f := plus_c(div_c(complex(sqr(Zq),0),Z_f_sqr,Errorcode),complex(-1,0));
    end else begin
      ze_f := complex(-1,0);
    end;
  end else begin
    ze_f := complex(-1,0);
  end;

  if GPars.rho_b > 0 then begin
    J_b := SLA.Jc(3,n_r,GPars);
    Z_b := sqrt_c(div_c(complex(GPars.rho_b * 1e3,0),J_b,Errorcode));
    xi_l := div_c(Z_b,Complex(Zq,0),Errorcode);
    if GPars.vers > 1e-22 then begin
      xi_l_no_Rou := xi_l;
      Z_b := SLA.ZL_Rou_Sh_Sma(Z_b,GPars.vers, GPars.asprat, GPars.rho_b * 1e3, om);
      xi_l := div_c(Z_b,Complex(Zq,0),Errorcode);
      xi_l := xi_l;
    end;
  end else begin
    xi_l := complex(0,0);
    xi_l_no_Rou := complex(0,0);
  end;
end;

procedure TPert.dfdf_Pe3(GPars : TPars; var dfdf,dfdf_Rou : TComplex; var Errorcode:Byte);
var coeffm1,coeff0,coeff1,coeff2,coeff_Rou,coeff_no_Rou,
    mueB,mueF,mufF,zeeB,zeeF,zefF,xilF,xilF_no_Rou : TComplex;
begin
  Assign_Pert(GPars,Errorcode);
  mueF := mu_e;
  mueB := complex(0,0);
//  mueB := mueF;   // Back-Electrode
  mufF := mu_f;
//  zeeB := complex(0,0);
  zeeF := ze_e;
  zeeB := zeeF; // BackElectrode
  zefF := ze_f;
  xilF := xi_l;
  xilF_no_Rou := xi_l_no_Rou;

{!(xilF - mueB xilF  - mueF xilF   - mufF xilF
+ mueB^2 xilF + mueF^2 xilF  +  mufF^2 xilF
+ 2 mueB mueF xilF  + 2 mueB mufF xilF + 2 mueF mufF xilF  + xilF^3/3)}
  coeffm1 := zero_c;
  coeffm1 := plus_c (coeffm1,                       powers_prod_c(xilF,1,one_c,0,one_c,0,   one_c,0,one_c,0));
  coeffm1 := plus_c (coeffm1,times_c(complex(-1,0), powers_prod_c(mueB,1,xilF,1,one_c,0,   one_c,0,one_c,0)));
  coeffm1 := plus_c (coeffm1,times_c(complex(-1,0), powers_prod_c(mueF,1,xilF,1,one_c,0,   one_c,0,one_c,0)));
  coeffm1 := plus_c (coeffm1,times_c(complex(-1,0), powers_prod_c(mufF,1,xilF,1,one_c,0,   one_c,0,one_c,0)));
  coeffm1 := plus_c (coeffm1,                       powers_prod_c(mueB,2,xilF,1,one_c,0,   one_c,0,one_c,0));
  coeffm1 := plus_c (coeffm1,                       powers_prod_c(mueF,2,xilF,1,one_c,0,   one_c,0,one_c,0));
  coeffm1 := plus_c (coeffm1,                        powers_prod_c(mufF,2,xilF,1,one_c,0,   one_c,0,one_c,0));
  coeffm1 := plus_c (coeffm1,times_c(complex(2,0),  powers_prod_c(mueB,1,mueF,1,xilF, 1,   one_c,0,one_c,0)));
  coeffm1 := plus_c (coeffm1,times_c(complex(2,0),  powers_prod_c(mueB,1,mufF,1,xilF, 1,   one_c,0,one_c,0)));
  coeffm1 := plus_c (coeffm1,times_c(complex(2,0),  powers_prod_c(mueF,1,mufF,1,xilF, 1,   one_c,0,one_c,0)));
  coeffm1 := plus_c (coeffm1,times_c(complex(1/3,0),powers_prod_c(xilF,3,one_c,0,one_c,0,   one_c,0,one_c,0)));
  coeffm1 := times_c(coeffm1,Im_c);
  coeffm1 := div_c  (coeffm1,Pi_c,Errorcode);

{((-mueB) + mueB^2 - mueB^3
   - mueF + 2 mueB mueF - 3 mueB^2 mueF
    + mueF^2 - 3 mueB mueF^2 - mueF^3
    - mufF + 2 mueB mufF - 3 mueB^2 mufF + 2 mueF mufF - 6 mueB mueF mufF -  3 mueF^2 mufF
    + mufF^2 - 3 mueB mufF^2 - 3 mueF mufF^2
    - mufF^3
    + mueF xilF^2 zeelF + mufF xilF^2 zefF)}

  coeff0 :=  zero_c;
  coeff0 := plus_c (coeff0,times_c(complex(-1 ,0),powers_prod_c(mueB,1,   one_c,0,   one_c,0,   one_c,0,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex(1  ,0),powers_prod_c(mueB,2,   one_c,0,   one_c,0,   one_c,0,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex(-1 ,0),powers_prod_c(mueB,3,   one_c,0,   one_c,0,   one_c,0,one_c,0)));

  coeff0 := plus_c (coeff0,times_c(complex(-1 ,0),powers_prod_c(mueF,1,   one_c,0,   one_c,0,   one_c,0,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex(2  ,0),powers_prod_c(mueB,1,   mueF ,1,   one_c,0,   one_c,0,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex(-3 ,0),powers_prod_c(mueB,2,   mueF ,1,   one_c,0,   one_c,0,one_c,0)));

  coeff0 := plus_c (coeff0,times_c(complex(1  ,0),powers_prod_c(one_c,0,   mueF ,2,   one_c,0,   one_c,0,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex(-3 ,0),powers_prod_c(mueB,1,   mueF ,2,   one_c,0,   one_c,0,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex(-1 ,0),powers_prod_c(one_c,0,   mueF ,3,   one_c,0,   one_c,0,one_c,0)));

  coeff0 := plus_c (coeff0,times_c(complex(-1 ,0),powers_prod_c(one_c,0,   mufF ,1,   one_c,0,   one_c,0,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex( 2 ,0),powers_prod_c(mueB,1,   mufF ,1,   one_c,0,   one_c,0,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex(-3 ,0),powers_prod_c(mueB,2,   mufF ,1,   one_c,0,   one_c,0,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex( 2 ,0),powers_prod_c(mueF,1,   mufF ,1,   one_c,0,   one_c,0,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex(-6 ,0),powers_prod_c(mueF,1,   mufF ,1,   mueB, 1,   one_c,0,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex(-3 ,0),powers_prod_c(mueF,2,   mufF ,1,   one_c,0,   one_c,0,one_c,0)));

  coeff0 := plus_c (coeff0,times_c(complex( 1 ,0),powers_prod_c(one_c,0,   mufF ,2,   one_c,0,   one_c,0,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex(-3 ,0),powers_prod_c(mueB,1,   mufF ,2,   one_c,0,   one_c,0,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex(-3 ,0),powers_prod_c(mueF,1,   mufF ,2,   one_c,0,   one_c,0,one_c,0)));

  coeff0 := plus_c (coeff0,times_c(complex(-1 ,0),powers_prod_c(one_c,0,   mufF ,3,   one_c,0,   one_c,0,one_c,0)));

  coeff0 := plus_c (coeff0,times_c(complex( 1 ,0),powers_prod_c(mueF,1,   xilF ,2,   zeeF, 1,   one_c,0,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex( 1 ,0),powers_prod_c(mufF,1,   xilF ,2,   zefF, 1,   one_c,0,one_c,0)));

{  !(mueF^2 xilF zeelF + 2 mueF mufF xilF zeelF + mufF^2 xilF zefF)}

  coeff1 :=  zero_c;
  coeff1 := plus_c (coeff1,times_c(complex(1,0),  powers_prod_c(mueF,2,xilF,1,zeeF,1,   one_c,0,one_c,0)));
  coeff1 := plus_c (coeff1,times_c(complex(2,0),  powers_prod_c(mueF,1,xilF,1,zeeF,1,   mufF, 1,one_c,0)));
  coeff1 := plus_c (coeff1,times_c(complex(1,0),  powers_prod_c(mufF,2,xilF,1,zefF,1,   one_c,0,one_c,0)));
  coeff1 := times_c(coeff1,Im_c);
  coeff1 := times_c(coeff1,Pi_c);

{!((-((mueB^3 zeeB)/3)) - (mueF^3 zeeF)/3 - (mufF^3 zefF)/3)
  - mueF^2 mufF zeeF - mueF mufF^2 zeeF}

  coeff2 :=  zero_c;
  coeff2 := plus_c (coeff2,times_c(complex(-1/3  ,0),powers_prod_c(mueB,3,zeeB,1,   one_c,0,   one_c,0,one_c,0)));
  coeff2 := plus_c (coeff2,times_c(complex(-1/3  ,0),powers_prod_c(mueF,3,zeeF,1,   one_c,0,   one_c,0,one_c,0)));
  coeff2 := plus_c (coeff2,times_c(complex(-1/3  ,0),powers_prod_c(mufF,3,zefF,1,   one_c,0,   one_c,0,one_c,0)));
  coeff2 := plus_c (coeff2,times_c(complex(-1    ,0),powers_prod_c(mueF,2,zeeF,1,   mufF,1,    one_c,0,one_c,0)));
  coeff2 := plus_c (coeff2,times_c(complex(-1    ,0),powers_prod_c(mueF,1,zeeF,1,   mufF,2,    one_c,0,one_c,0)));
  coeff2 := times_c(coeff2,Pi_c);
  coeff2 := times_c(coeff2,Pi_c);

  dfdf := zero_c;
  dfdf := plus_c(dfdf,Div_c  (coeffm1,n_c,Errorcode));
  dfdf := plus_c(dfdf,        coeff0       );
  dfdf := plus_c(dfdf,times_c(coeff1, n_c));
  dfdf := plus_c(dfdf,times_c(coeff2, Power_c(n_c,2)));
////
  coeff_Rou := xilF;
  coeff_Rou := plus_c (coeff_Rou,times_c(complex(1/3,0),powers_prod_c(xilF,3,one_c,0,one_c,0,   one_c,0,one_c,0)));
  coeff_no_Rou := xilF_no_Rou;
  coeff_no_Rou := plus_c (coeff_no_Rou,times_c(complex(1/3,0),powers_prod_c(xilF_no_Rou,3,one_c,0,one_c,0,   one_c,0,one_c,0)));
  dfdf_Rou := minus_c(coeff_Rou,coeff_no_Rou);
  dfdf_Rou := times_c(dfdf_Rou,Im_c);
  dfdf_Rou := div_c  (dfdf_Rou,Pi_c,Errorcode);
  dfdf_Rou := div_c  (dfdf_Rou,n_c,Errorcode);
end;

procedure TPert.dfstar_Pe3(var df_Pert,dG_Pert,df_Rou,dG_Rou : Double; var Errorcode:Byte);
var df_Pert_raw,dG_Pert_raw,df_Pert_ref,dG_Pert_ref : Double;
    dfdf,dfdf_Rou_Raw,dfdf_Rou_Ref : TComplex;
begin
  dfdf_Pe3(Pars,dfdf,dfdf_Rou_Raw,Errorcode);
  df_Pert_raw := dfdf.real * n_r * f_fund;
  dG_Pert_raw := dfdf.imag * n_r * f_fund;
  dfdf_Pe3(Ref_Pars,dfdf,dfdf_Rou_Ref,Errorcode);
  df_Pert_ref := dfdf.real * n_r * f_fund;
  dG_Pert_ref := dfdf.imag * n_r * f_fund;
  df_Pert := df_Pert_raw - df_Pert_ref;
  dG_Pert := dG_Pert_raw - dG_Pert_ref;
  df_Rou := (dfdf_Rou_Raw.real - dfdf_Rou_Ref.real) * n_r * f_fund;
  dG_Rou := (dfdf_Rou_Raw.imag - dfdf_Rou_Ref.imag) * n_r * f_fund;
end;

procedure TPert.dfdf_Pe5(GPars : TPars; var dfdf,dfdf_Rou : TComplex; var Errorcode:Byte);
var coeffm1,coeff0,coeff1,coeff2,coeff3,coeff4,coeff_Rou,coeff_no_Rou,
    mueB,mueF,mufF,zeeB,zeeF,zefF,xilF,xilF_no_Rou : TComplex;
begin
  Assign_Pert(GPars,Errorcode);
  mueF := mu_e;
  mueB := complex(0,0);
//  mueB := mueF;   // Back-Electrode
  mufF := mu_f;
//  zeeB := complex(0,0);
  zeeF := ze_e;
  zeeB := zeeF;// BackElectrode
  zefF := ze_f;
  xilF := xi_l;
  xilF_no_Rou := xi_l_no_Rou;
  {from Pertur_binang_Feb_04_5th_Order_C.nb }

    {coeffm1
   xilF - mueB xilF + mueB^2 xilF - mueB^3 xilF + mueB^4 xilF -
    mueF xilF + 2 mueB mueF xilF - 3 mueB^2 mueF xilF +
    4 mueB^3 mueF xilF + mueF^2 xilF - 3 mueB mueF^2 xilF +
    6 mueB^2 mueF^2 xilF - mueF^3 xilF + 4 mueB mueF^3 xilF +
    mueF^4 xilF - mufF xilF + 2 mueB mufF xilF -
    3 mueB^2 mufF xilF + 4 mueB^3 mufF xilF + 2 mueF mufF xilF -
    6 mueB mueF mufF xilF + 12 mueB^2 mueF mufF xilF -
    3 mueF^2 mufF xilF + 12 mueB mueF^2 mufF xilF +
    4 mueF^3 mufF xilF + mufF^2 xilF - 3 mueB mufF^2 xilF +
    6 mueB^2 mufF^2 xilF - 3 mueF mufF^2 xilF +
    12 mueB mueF mufF^2 xilF + 6 mueF^2 mufF^2 xilF -
    mufF^3 xilF + 4 mueB mufF^3 xilF + 4 mueF mufF^3 xilF +
    mufF^4 xilF +
    xilF^3/3 - (mueB xilF^3)/3 + (mueB^2 xilF^3)/3 - (mueF
xilF^3)/3 +
    2/3 mueB mueF xilF^3 + (mueF^2 xilF^3)/3 - (mufF
xilF^3)/3 + 2/3 mueB mufF xilF^3 +
    2/3 mueF mufF xilF^3 + (mufF^2 xilF^3)/3 + xilF^5/5 +
    mueF xilF^3 zeeF - 2 mueB mueF xilF^3 zeeF -
    3 mueF^2 xilF^3 zeeF - 4 mueF mufF xilF^3 zeeF +
    mufF xilF^3 zefF - 2 mueB mufF xilF^3 zefF -
    2 mueF mufF xilF^3 zefF - 3 mufF^2 xilF^3 zefF         }


{    xilF - mueB xilF + mueB^2 xilF - mueB^3 xilF + mueB^4 xilF
    - mueF xilF + 2 mueF xilF mueB - 3 mueF xilF mueB^2  +  4 mueF xilF mueB^3
    + mueF^2 xilF - 3 mueF^2 xilF mueB + 6 mueF^2 xilF mueB^2
    - mueF^3 xilF + 4 mueF^3 xilF mueB
    + mueF^4 xilF
    - mufF xilF + 2 mufF xilF mueB - 3 mufF xilF mueB^2 + 4 mufF xilF mueB^3
    + 2 mueF mufF xilF - 6 mueF mufF xilF mueB  + 12 mueF mufF xilF mueB^2 - 3 mueF^2 mufF xilF
    + 12 mueB mueF^2 mufF xilF + 4 mueF^3 mufF xilF
    + mufF^2 xilF - 3 mufF^2 xilF mueB + 6 mufF^2 xilF mueB^2 - 3 mufF^2 xilF mueF
    + 12 mufF^2 xilF mueB mueF + 6 mufF^2 xilF mueF^2
    - mufF^3 xilF + 4 mufF^3 xilF mueB + 4 mufF^3 xilF mueF
    + mufF^4 xilF
    + xilF^3/3 - (xilF^3 mueB )/3 + (xilF^3 mueB^2 )/3 - (xilF^3 mueF)/3
    + 2/3 xilF^3 mueB mueF + (xilF^3 mueF^2)/3 - (xilF^3 mufF)/3 + 2/3 xilF^3 mueB mufF
    + 2/3 xilF^3 mueF mufF + (xilF^3 mufF^2 )/3
    + xilF^5/5
    + mueF xilF^3 zeeF     - 2 mueF xilF^3 zeeF mueB
    - 3 mueF^2 xilF^3 zeeF - 4 mueF xilF^3 zeeF mufF
    + mufF xilF^3 zefF - 2 mufF xilF^3 zefF mueB
    - 2 mufF xilF^3 zefF mueF - 3 mufF^2 xilF^3 zefF }

  coeffm1 :=  zero_c;

    {xilF - mueB xilF + mueB^2 xilF - mueB^3 xilF + mueB^4 xilF                                    }
  coeffm1 := plus_c (coeffm1,times_c(complex( 1,0), powers_prod_c(one_c,0,xilF,1,   one_c,0,   one_c,0,one_c,0)));
  coeffm1 := plus_c (coeffm1,times_c(complex(-1,0), powers_prod_c(mueB,1,xilF,1,   one_c,0,   one_c,0,one_c,0)));
  coeffm1 := plus_c (coeffm1,times_c(complex( 1,0), powers_prod_c(mueB,2,xilF,1,   one_c,0,   one_c,0,one_c,0)));
  coeffm1 := plus_c (coeffm1,times_c(complex(-1,0), powers_prod_c(mueB,3,xilF,1,   one_c,0,   one_c,0,one_c,0)));
  coeffm1 := plus_c (coeffm1,times_c(complex( 1,0), powers_prod_c(mueB,4,xilF,1,   one_c,0,   one_c,0,one_c,0)));

    {- mueF xilF + 2 mueF xilF mueB - 3 mueF xilF mueB^2  +  4 mueF xilF mueB^3                    }
  coeffm1 := plus_c (coeffm1,times_c(complex(-1,0), powers_prod_c(mueF,1,xilF,1,   one_c,0,   one_c,0,one_c,0)));
  coeffm1 := plus_c (coeffm1,times_c(complex( 2,0), powers_prod_c(mueF,1,xilF,1,   mueB ,1,   one_c,0,one_c,0)));
  coeffm1 := plus_c (coeffm1,times_c(complex(-3,0), powers_prod_c(mueF,1,xilF,1,   mueB ,2,   one_c,0,one_c,0)));
  coeffm1 := plus_c (coeffm1,times_c(complex( 4,0), powers_prod_c(mueF,1,xilF,1,   mueB ,3,   one_c,0,one_c,0)));

    {+ mueF^2 xilF - 3 mueF^2 xilF mueB + 6 mueF^2 xilF mueB^2                                     }
  coeffm1 := plus_c (coeffm1,times_c(complex( 1,0), powers_prod_c(mueF,2,xilF,1,   one_c,0,   one_c,0,one_c,0)));
  coeffm1 := plus_c (coeffm1,times_c(complex(-3,0), powers_prod_c(mueF,2,xilF,1,   mueB ,1,   one_c,0,one_c,0)));
  coeffm1 := plus_c (coeffm1,times_c(complex( 6,0), powers_prod_c(mueF,2,xilF,1,   mueB ,2,   one_c,0,one_c,0)));

    {- mueF^3 xilF + 4 mueF^3 xilF mueB                                                            }
  coeffm1 := plus_c (coeffm1,times_c(complex(-1,0), powers_prod_c(mueF,3,xilF,1,   one_c,0,   one_c,0,one_c,0)));
  coeffm1 := plus_c (coeffm1,times_c(complex( 4,0), powers_prod_c(mueF,3,xilF,1,   mueB ,1,   one_c,0,one_c,0)));

    {+ mueF^4 xilF                                                                                 }
  coeffm1 := plus_c (coeffm1,times_c(complex( 1,0), powers_prod_c(mueF,4,xilF,1,   one_c,0,   one_c,0,one_c,0)));

    {- mufF xilF + 2 mufF xilF mueB - 3 mufF xilF mueB^2 + 4 mufF xilF mueB^3                      }
  coeffm1 := plus_c (coeffm1,times_c(complex(-1,0), powers_prod_c(mufF,1,xilF,1,   one_c,0,   one_c,0,one_c,0)));
  coeffm1 := plus_c (coeffm1,times_c(complex( 2,0), powers_prod_c(mufF,1,xilF,1,   mueB ,1,   one_c,0,one_c,0)));
  coeffm1 := plus_c (coeffm1,times_c(complex(-3,0), powers_prod_c(mufF,1,xilF,1,   mueB ,2,   one_c,0,one_c,0)));
  coeffm1 := plus_c (coeffm1,times_c(complex( 4,0), powers_prod_c(mufF,1,xilF,1,   mueB ,3,   one_c,0,one_c,0)));

    {+ 2 mueF mufF xilF - 6 mueF mufF xilF mueB  + 12 mueF mufF xilF mueB^2 - 3 mueF^2 mufF xilF   }
  coeffm1 := plus_c (coeffm1,times_c(complex( 2,0), powers_prod_c(mueF,1,xilF,1,   mufF,1,   one_c,0,one_c,0)));
  coeffm1 := plus_c (coeffm1,times_c(complex(-6,0), powers_prod_c(mueF,1,xilF,1,   mufF,1,   mueB ,1,one_c,0)));
  coeffm1 := plus_c (coeffm1,times_c(complex(12,0), powers_prod_c(mueF,1,xilF,1,   mufF,1,   mueB ,2,one_c,0)));
  coeffm1 := plus_c (coeffm1,times_c(complex(-3,0), powers_prod_c(mueF,2,xilF,1,   mufF,1,   one_c,0,one_c,0)));

    {+ 12 mueB mueF^2 mufF xilF + 4 mueF^3 mufF xilF                                               }
  coeffm1 := plus_c (coeffm1,times_c(complex(12,0), powers_prod_c(mufF,1,xilF,1,   mueF,2,   mueB ,1,one_c,0)));
  coeffm1 := plus_c (coeffm1,times_c(complex( 4,0), powers_prod_c(mufF,1,xilF,1,   mueF,3,   one_c,0,one_c,0)));

    {+ mufF^2 xilF - 3 mufF^2 xilF mueB + 6 mufF^2 xilF mueB^2 - 3 mufF^2 xilF mueF                }
    {+ 12 mufF^2 xilF mueB mueF + 6 mufF^2 xilF mueF^2                                             }
  coeffm1 := plus_c (coeffm1,times_c(complex( 1,0), powers_prod_c(mufF,2,xilF,1,   one_c,0,   one_c,0,one_c,0)));
  coeffm1 := plus_c (coeffm1,times_c(complex(-3,0), powers_prod_c(mufF,2,xilF,1,   mueB ,1,   one_c,0,one_c,0)));
  coeffm1 := plus_c (coeffm1,times_c(complex( 6,0), powers_prod_c(mufF,2,xilF,1,   mueB ,2,   one_c,0,one_c,0)));
  coeffm1 := plus_c (coeffm1,times_c(complex(-3,0), powers_prod_c(mufF,2,xilF,1,   mueF ,1,   one_c,0,one_c,0)));
  coeffm1 := plus_c (coeffm1,times_c(complex(12,0), powers_prod_c(mufF,2,xilF,1,   mueF ,1,   mueB ,1,one_c,0)));
  coeffm1 := plus_c (coeffm1,times_c(complex( 6,0), powers_prod_c(mufF,2,xilF,1,   mueF ,2,   one_c,0,one_c,0)));

    {- mufF^3 xilF + 4 mufF^3 xilF mueB + 4 mufF^3 xilF mueF                                       }
  coeffm1 := plus_c (coeffm1,times_c(complex(-1,0), powers_prod_c(mufF,3,xilF,1,   one_c,0,   one_c,0,one_c,0)));
  coeffm1 := plus_c (coeffm1,times_c(complex( 4,0), powers_prod_c(mufF,3,xilF,1,   mueB ,1,   one_c,0,one_c,0)));
  coeffm1 := plus_c (coeffm1,times_c(complex( 4,0), powers_prod_c(mufF,3,xilF,1,   mueF ,1,   one_c,0,one_c,0)));

    {+ mufF^4 xilF                                                                                 }
  coeffm1 := plus_c (coeffm1,times_c(complex( 4,0), powers_prod_c(mufF,4,xilF,1,   one_c,0,   one_c,0,one_c,0)));

    {+ xilF^3/3 - (xilF^3 mueB )/3 + (xilF^3 mueB^2 )/3 - (xilF^3 mueF)/3                          }
  coeffm1 := plus_c (coeffm1,times_c(complex( 1/3,0),powers_prod_c(one_c,0,xilF,3,   one_c,0,   one_c,0,one_c,0)));
  coeffm1 := plus_c (coeffm1,times_c(complex(-1/3,0),powers_prod_c(one_c,0,xilF,3,   mueB ,1,   one_c,0,one_c,0)));
  coeffm1 := plus_c (coeffm1,times_c(complex( 1/3,0),powers_prod_c(one_c,0,xilF,3,   mueB ,2,   one_c,0,one_c,0)));
  coeffm1 := plus_c (coeffm1,times_c(complex(-1/3,0),powers_prod_c(one_c,0,xilF,3,   mueF ,1,   one_c,0,one_c,0)));

    {+ 2/3 xilF^3 mueB mueF + (xilF^3 mueF^2)/3}
  coeffm1 := plus_c (coeffm1,times_c(complex( 2/3,0),powers_prod_c(mueF,1,xilF,3,   mueB ,1,   one_c,0,one_c,0)));
  coeffm1 := plus_c (coeffm1,times_c(complex( 1/3,0),powers_prod_c(mueF,2,xilF,3,   one_c,0,   one_c,0,one_c,0)));

    {- (xilF^3 mufF)/3 + 2/3 xilF^3 mueB mufF                                                       }
  coeffm1 := plus_c (coeffm1,times_c(complex(-1/3,0),powers_prod_c(mufF,1,xilF,3,   one_c,0,   one_c,0,one_c,0)));
  coeffm1 := plus_c (coeffm1,times_c(complex( 2/3,0),powers_prod_c(mufF,1,xilF,3,   mueB ,1,   one_c,0,one_c,0)));

    {+ 2/3 xilF^3 mueF mufF + (xilF^3 mufF^2 )/3                                                   }
  coeffm1 := plus_c (coeffm1,times_c(complex( 2/3,0),powers_prod_c(mufF,1,xilF,3,   mueF ,1,   one_c,0,one_c,0)));
  coeffm1 := plus_c (coeffm1,times_c(complex( 1/3,0),powers_prod_c(mufF,2,xilF,3,   one_c,0,   one_c,0,one_c,0)));

    {+ xilF^5/5                                                                                    }
  coeffm1 := plus_c (coeffm1,times_c(complex( 1/5,0),powers_prod_c(one_c,0,xilF,5,   one_c,0,   one_c,0,one_c,0)));

    {+ mueF xilF^3 zeeF     - 2 mueF xilF^3 zeeF mueB                                              }
  coeffm1 := plus_c (coeffm1,times_c(complex( 1,0), powers_prod_c(mueF,1,xilF,3,   zeeF,1,   one_c,0,one_c,0)));
  coeffm1 := plus_c (coeffm1,times_c(complex(-2,0), powers_prod_c(mueF,1,xilF,3,   zeeF,1,   mueB ,1,one_c,0)));

    {- 3 mueF^2 xilF^3 zeeF - 4 mueF xilF^3 zeeF mufF                                              }
  coeffm1 := plus_c (coeffm1,times_c(complex(-3,0), powers_prod_c(mueF,2,xilF,3,   zeeF,1,   one_c,0,one_c,0)));
  coeffm1 := plus_c (coeffm1,times_c(complex(-4,0), powers_prod_c(mueF,1,xilF,3,   zeeF,1,   mufF, 1,one_c,0)));

    {+ mufF xilF^3 zefF - 2 mufF xilF^3 zefF mueB                                                  }
  coeffm1 := plus_c (coeffm1,times_c(complex( 1,0), powers_prod_c(mufF,1,xilF,3,   zefF,1,   one_c,0,one_c,0)));
  coeffm1 := plus_c (coeffm1,times_c(complex(-2,0), powers_prod_c(mufF,1,xilF,3,   zefF,1,   mueB ,1,one_c,0)));


    {- 2 mufF xilF^3 zefF mueF - 3 mufF^2 xilF^3 zefF                                              }
  coeffm1 := plus_c (coeffm1,times_c(complex(-2,0), powers_prod_c(mufF,1,xilF,3,   zefF,1,   mueF ,1,one_c,0)));
  coeffm1 := plus_c (coeffm1,times_c(complex(-3,0), powers_prod_c(mufF,2,xilF,3,   zefF,1,   one_c,0,one_c,0)));

  coeffm1 := times_c(coeffm1,Im_c);
  coeffm1 := div_c  (coeffm1,Pi_c,Errorcode);


{   -mueB + mueB^2 - mueB^3 + mueB^4 - mueB^5 - mueF +
    2 mueB mueF - 3 mueB^2 mueF + 4 mueB^3 mueF - 5 mueB^4 mueF +
    mueF^2 - 3 mueB mueF^2 + 6 mueB^2 mueF^2 - 10 mueB^3 mueF^2 -
    mueF^3 + 4 mueB mueF^3 - 10 mueB^2 mueF^3 + mueF^4 -
    5 mueB mueF^4 - mueF^5 - mufF + 2 mueB mufF - 3 mueB^2 mufF +
    4 mueB^3 mufF - 5 mueB^4 mufF + 2 mueF mufF -
    6 mueB mueF mufF + 12 mueB^2 mueF mufF - 20 mueB^3 mueF mufF -
    3 mueF^2 mufF + 12 mueB mueF^2 mufF - 30 mueB^2 mueF^2 mufF +
    4 mueF^3 mufF - 20 mueB mueF^3 mufF - 5 mueF^4 mufF + mufF^2 -
    3 mueB mufF^2 + 6 mueB^2 mufF^2 - 10 mueB^3 mufF^2 -
    3 mueF mufF^2 + 12 mueB mueF mufF^2 - 30 mueB^2 mueF mufF^2 +
    6 mueF^2 mufF^2 - 30 mueB mueF^2 mufF^2 - 10 mueF^3 mufF^2 -
    mufF^3 + 4 mueB mufF^3 - 10 mueB^2 mufF^3 + 4 mueF mufF^3 -
    20 mueB mueF mufF^3 - 10 mueF^2 mufF^3 + mufF^4 -
    5 mueB mufF^4 - 5 mueF mufF^4 - mufF^5 + mueB^3 xilF^2 zeeB +
    mueF xilF^2 zeeF - 2 mueB mueF xilF^2 zeeF +
    3 mueB^2 mueF xilF^2 zeeF - 4 mueF^2 xilF^2 zeeF +
    12 mueB mueF^2 xilF^2 zeeF + 10 mueF^3 xilF^2 zeeF -
    6 mueF mufF xilF^2 zeeF + 18 mueB mueF mufF xilF^2 zeeF +
    27 mueF^2 mufF xilF^2 zeeF + 18 mueF mufF^2 xilF^2 zeeF +
    mueF xilF^4 zeeF + mufF xilF^2 zefF -
    2 mueB mufF xilF^2 zefF + 3 mueB^2 mufF xilF^2 zefF -
    2 mueF mufF xilF^2 zefF + 6 mueB mueF mufF xilF^2 zefF +
    3 mueF^2 mufF xilF^2 zefF - 4 mufF^2 xilF^2 zefF +
    12 mueB mufF^2 xilF^2 zefF + 12 mueF mufF^2 xilF^2 zefF +
    10 mufF^3 xilF^2 zefF + mufF xilF^4 zefF}

{    - mueB + mueB^2 - mueB^3 + mueB^4 - mueB^5
    - mueF + 2 mueB mueF - 3 mueB^2 mueF + 4 mueB^3 mueF - 5 mueB^4 mueF
    + mueF^2 - 3 mueB mueF^2 + 6 mueB^2 mueF^2 - 10 mueB^3 mueF^2
    - mueF^3 + 4 mueB mueF^3 - 10 mueB^2 mueF^3
    + mueF^4 - 5 mueB mueF^4
    - mueF^5
    - mufF + 2 mueB mufF - 3 mueB^2 mufF + 4 mueB^3 mufF - 5 mueB^4 mufF
    + 2 mueF mufF - 6 mueF mufF mueB + 12 mueF mufF mueB^2 - 20 mueF mufF mueB^3
    - 3 mueF^2 mufF + 12 mueF^2 mufF mueB - 30 mueF^2 mufF mueB^2
    + 4 mueF^3 mufF - 20 mueF^3 mufF mueB
    - 5 mueF^4 mufF
    + mufF^2 - 3 mueB mufF^2 + 6 mueB^2 mufF^2 - 10 mueB^3 mufF^2
    - 3 mueF mufF^2 + 12 mueB mueF mufF^2 - 30 mueB^2 mueF mufF^2
    + 6 mueF^2 mufF^2 - 30 mueF^2 mufF^2 mueB
    - 10 mueF^3 mufF^2
    - mufF^3 + 4 mueB mufF^3 - 10 mueB^2 mufF^3
    + 4 mueF mufF^3 - 20 mueB mueF mufF^3 - 10 mueF^2 mufF^3
    + mufF^4 - 5 mueB mufF^4 - 5 mueF mufF^4
    - mufF^5
    + mueB^3 xilF^2 zeeB
    + mueF xilF^2 zeeF - 2 mueF xilF^2 zeeF mueB
    + mueF xilF^2 zeeF 3 mueB^2 - 4 mueF^2 xilF^2 zeeF
    + 12 mueF^2 xilF^2 zeeF mueB + 10 mueF^3 xilF^2 zeeF
    - 6  mueF   xilF^2 zeeF mufF + 18 mueF xilF^2 zeeF mueB mufF
    + 27 mueF^2 xilF^2 zeeF mufF + 18 mueF xilF^2 zeeF mufF^2
    +    mueF   xilF^4 zeeF
    +    mufF   xilF^2 zefF - 2 mufF xilF^2 zefF mueB + 3 mufF xilF^2 zefF mueB^2
    -  2 mufF   xilF^2 zefF mueF   + 6 mufF   xilF^2 zefF mueB mueF
    +  3 mufF   xilF^2 zefF mueF^2 - 4 mufF^2 xilF^2 zefF
    + 12 mufF^2 xilF^2 zefF mueB  + 12 mufF^2 xilF^2 zefF mueF
    + 10 mufF^3 xilF^2 zefF       + mufF xilF^4 zefF}

{check}
  coeff0 :=  zero_c;
    {- mueB + mueB^2 - mueB^3 + mueB^4 - mueB^5                                     }
  coeff0 := plus_c (coeff0,times_c(complex(-1,0), powers_prod_c(mueB,1,one_c,0,   one_c,0,   one_c,0,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex( 1,0), powers_prod_c(mueB,2,one_c,0,   one_c,0,   one_c,0,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex(-1,0), powers_prod_c(mueB,3,one_c,0,   one_c,0,   one_c,0,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex( 1,0), powers_prod_c(mueB,4,one_c,0,   one_c,0,   one_c,0,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex(-1,0), powers_prod_c(mueB,5,one_c,0,   one_c,0,   one_c,0,one_c,0)));

    {- mueF + 2 mueB mueF - 3 mueB^2 mueF + 4 mueB^3 mueF - 5 mueB^4 mueF           }
  coeff0 := plus_c (coeff0,times_c(complex(-1,0), powers_prod_c(one_c,0,mueF,1,   one_c,0,   one_c,0,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex( 2,0), powers_prod_c(mueB,1,mueF,1,   one_c,0,   one_c,0,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex(-3,0), powers_prod_c(mueB,2,mueF,1,   one_c,0,   one_c,0,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex( 4,0), powers_prod_c(mueB,3,mueF,1,   one_c,0,   one_c,0,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex(-5,0), powers_prod_c(mueB,4,mueF,1,   one_c,0,   one_c,0,one_c,0)));

    {+ mueF^2 - 3 mueB mueF^2 + 6 mueB^2 mueF^2 - 10 mueB^3 mueF^2                  }
  coeff0 := plus_c (coeff0,times_c(complex(  1,0),powers_prod_c(one_c,0,mueF,2,   one_c,0,   one_c,0,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex( -3,0),powers_prod_c(mueB,1,mueF,2,   one_c,0,   one_c,0,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex(  6,0),powers_prod_c(mueB,2,mueF,2,   one_c,0,   one_c,0,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex(-10,0),powers_prod_c(mueB,3,mueF,2,   one_c,0,   one_c,0,one_c,0)));

    {- mueF^3 + 4 mueB mueF^3 - 10 mueB^2 mueF^3                                    }
  coeff0 := plus_c (coeff0,times_c(complex( -1,0),powers_prod_c(one_c,0,mueF,3,   one_c,0,   one_c,0,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex(  4,0),powers_prod_c(mueB,1,mueF,3,   one_c,0,   one_c,0,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex(-10,0),powers_prod_c(mueB,2,mueF,3,   one_c,0,   one_c,0,one_c,0)));

    {+ mueF^4 - 5 mueB mueF^4                                                       }
  coeff0 := plus_c (coeff0,times_c(complex(  1,0),powers_prod_c(one_c,0,mueF,4,   one_c,0,   one_c,0,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex( -5,0),powers_prod_c(mueB,1,mueF,4,   one_c,0,   one_c,0,one_c,0)));

    {- mueF^5                                                                       }
  coeff0 := plus_c (coeff0,times_c(complex( -1,0),powers_prod_c(one_c,0,mueF,5,   one_c,0,   one_c,0,one_c,0)));

    {- mufF + 2 mueB mufF - 3 mueB^2 mufF + 4 mueB^3 mufF - 5 mueB^4 mufF           }
  coeff0 := plus_c (coeff0,times_c(complex(-1,0), powers_prod_c(one_c,0,mufF,1,   one_c,0,   one_c,0,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex( 2,0), powers_prod_c(mueB,1,mufF,1,   one_c,0,   one_c,0,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex(-3,0), powers_prod_c(mueB,2,mufF,1,   one_c,0,   one_c,0,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex( 4,0), powers_prod_c(mueB,3,mufF,1,   one_c,0,   one_c,0,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex(-5,0), powers_prod_c(mueB,4,mufF,1,   one_c,0,   one_c,0,one_c,0)));

    {+ 2 mueF mufF - 6 mueF mufF mueB + 12 mueF mufF mueB^2 - 20 mueF mufF mueB^3   }
  coeff0 := plus_c (coeff0,times_c(complex(  2,0),powers_prod_c(one_c,0,mufF,1,   mueF, 1,   one_c,0,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex( -6,0),powers_prod_c(mueB,1,mufF,1,   mueF, 1,   one_c,0,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex( 12,0),powers_prod_c(mueB,2,mufF,1,   mueF, 1,   one_c,0,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex(-20,0),powers_prod_c(mueB,3,mufF,1,   mueF, 1,   one_c,0,one_c,0)));

    {- 3 mueF^2 mufF + 12 mueF^2 mufF mueB - 30 mueF^2 mufF mueB^2                  }
  coeff0 := plus_c (coeff0,times_c(complex( -3,0),powers_prod_c(one_c,0,mufF,1,   mueF, 2,   one_c,0,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex( 12,0),powers_prod_c(mueB,1,mufF,1,   mueF, 2,   one_c,0,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex(-30,0),powers_prod_c(mueB,2,mufF,1,   mueF, 2,   one_c,0,one_c,0)));

    {+ 4 mueF^3 mufF - 20 mueF^3 mufF mueB                                          }
  coeff0 := plus_c (coeff0,times_c(complex(  4,0),powers_prod_c(one_c,0,mufF,1,   mueF, 3,   one_c,0,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex(-20,0),powers_prod_c(mueB,1,mufF,1,   mueF, 3,   one_c,0,one_c,0)));

    {- 5 mueF^4 mufF                                                                }
  coeff0 := plus_c (coeff0,times_c(complex( -5,0),powers_prod_c(one_c,0,mufF,1,   mueF, 4,   one_c,0,one_c,0)));

    {+ mufF^2 - 3 mueB mufF^2 + 6 mueB^2 mufF^2 - 10 mueB^3 mufF^2                  }
  coeff0 := plus_c (coeff0,times_c(complex(  1,0),powers_prod_c(one_c,0,mufF,2,   mueF, 1,   one_c,0,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex( -3,0),powers_prod_c(mueB,1,mufF,2,   mueF, 1,   one_c,0,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex(  6,0),powers_prod_c(mueB,2,mufF,2,   mueF, 1,   one_c,0,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex(-10,0),powers_prod_c(mueB,3,mufF,2,   mueF, 1,   one_c,0,one_c,0)));

    {- 3 mueF mufF^2 + 12 mueB mueF mufF^2 - 30 mueB^2 mueF mufF^2                  }
  coeff0 := plus_c (coeff0,times_c(complex( -3,0),powers_prod_c(one_c,0,mufF,2,   mueF, 1,   one_c,0,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex( 12,0),powers_prod_c(mueB,1,mufF,2,   mueF, 1,   one_c,0,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex(-30,0),powers_prod_c(mueB,2,mufF,2,   mueF, 1,   one_c,0,one_c,0)));

    {+ 6 mueF^2 mufF^2 - 30 mueF^2 mufF^2 mueB                                      }
  coeff0 := plus_c (coeff0,times_c(complex(  6,0),powers_prod_c(one_c,0,mufF,2,   mueF, 2,   one_c,0,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex(-30,0),powers_prod_c(mueB,1,mufF,2,   mueF, 2,   one_c,0,one_c,0)));

    {- 10 mueF^3 mufF^2                                                             }
  coeff0 := plus_c (coeff0,times_c(complex(-10,0),powers_prod_c(one_c,0,mufF,2,   mueF, 3,   one_c,0,one_c,0)));

    {- mufF^3 + 4 mueB mufF^3 - 10 mueB^2 mufF^3                     }
  coeff0 := plus_c (coeff0,times_c(complex( -1,0),powers_prod_c(one_c,0,mufF,3,   one_c,0,   one_c,0,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex(  4,0),powers_prod_c(mueB,1,mufF,3,   one_c,0,   one_c,0,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex(-10,0),powers_prod_c(mueB,2,mufF,3,   one_c,0,   one_c,0,one_c,0)));

    {+ 4 mueF mufF^3 - 20 mueB mueF mufF^3 - 10 mueF^2 mufF^3                              }
  coeff0 := plus_c (coeff0,times_c(complex(  4,0),powers_prod_c(one_c,0,mufF,3,   mueF, 1,   one_c,0,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex(-20,0),powers_prod_c(mueB,1,mufF,3,   mueF, 1,   one_c,0,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex(-10,0),powers_prod_c(one_c,0,mufF,3,   mueF, 2,   one_c,0,one_c,0)));

    { + mufF^4 - 5 mueB mufF^4 - 5 mueF mufF^4                                       }
  coeff0 := plus_c (coeff0,times_c(complex(  1,0),powers_prod_c(one_c,0,mufF,4,   one_c,0,   one_c,0,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex( -5,0),powers_prod_c(mueB,1,mufF,4,   one_c,0,   one_c,0,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex( -5,0),powers_prod_c(mueF,1,mufF,4,   one_c,0,   one_c,0,one_c,0)));

    {- mufF^5                                       }
  coeff0 := plus_c (coeff0,times_c(complex( -1,0),powers_prod_c(one_c,0,mufF,5,   one_c,0,   one_c,0,one_c,0)));

    {+ mueB^3 xilF^2 zeeB                                                           }
  coeff0 := plus_c (coeff0,times_c(complex(  1,0),powers_prod_c(mueB,3,xilF,2,   zeeB, 1,   one_c,0,one_c,0)));

    {+ mueF xilF^2 zeeF - 2 mueF xilF^2 zeeF mueB                                   }
  coeff0 := plus_c (coeff0,times_c(complex(  1,0),powers_prod_c(one_c,0,xilF,2,   zeeF, 1,   mueF, 1,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex( -2,0),powers_prod_c(mueB,1,xilF,2,   zeeF, 1,   mueF, 1,one_c,0)));

    {+ mueF xilF^2 zeeF 3 mueB^2 - 4 mueF^2 xilF^2 zeeF                             }
  coeff0 := plus_c (coeff0,times_c(complex(  3,0),powers_prod_c(mueB,2,xilF,2,   zeeF, 1,   mueF, 1,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex( -4,0),powers_prod_c(one_c,0,xilF,2,   zeeF, 1,   mueF, 2,one_c,0)));

    {+ 12 mueF^2 xilF^2 zeeF mueB + 10 mueF^3 xilF^2 zeeF                           }
  coeff0 := plus_c (coeff0,times_c(complex( 12,0),powers_prod_c(mueB,1,xilF,2,   zeeF, 1,   mueF, 2,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex( 10,0),powers_prod_c(one_c,0,xilF,2,   zeeF, 1,   mueF, 3,one_c,0)));

    {- 6  mueF   xilF^2 zeeF mufF + 18 mueF xilF^2 zeeF mueB mufF                   }
  coeff0 := plus_c (coeff0,times_c(complex( -6,0),powers_prod_c(mueF,1,xilF,2,   zeeF, 1,   mufF, 1,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex( 18,0),powers_prod_c(mueF,1,xilF,2,   zeeF, 1,   mufF, 1,mueB, 1)));

    {+ 27 mueF^2 xilF^2 zeeF mufF + 18 mueF xilF^2 zeeF mufF^2                      }
  coeff0 := plus_c (coeff0,times_c(complex( 27,0),powers_prod_c(mueF,2,xilF,2,   zeeF, 1,   mufF, 1,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex( 18,0),powers_prod_c(mueF,1,xilF,2,   zeeF, 1,   mufF, 2,one_c,0)));

    {+    mueF   xilF^4 zeeF                                                       }
  coeff0 := plus_c (coeff0,times_c(complex(  1,0),powers_prod_c(mueF,1,xilF,4,   zeeF, 1,   one_c,0,one_c,0)));


    {+    mufF   xilF^2 zefF - 2 mufF xilF^2 zefF mueB + 3 mufF xilF^2 zefF mueB^2 }
  coeff0 := plus_c (coeff0,times_c(complex(  1,0),powers_prod_c(one_c,0,xilF,2,   zefF, 1,   mufF, 1,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex( -2,0),powers_prod_c(mueB, 1,xilF,2,   zefF, 1,   mufF, 1,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex(  3,0),powers_prod_c(mueB, 2,xilF,2,   zefF, 1,   mufF, 1,one_c,0)));


    {-  2 mufF   xilF^2 zefF mueF   + 6 mufF   xilF^2 zefF mueB mueF                }
  coeff0 := plus_c (coeff0,times_c(complex( -2,0),powers_prod_c(one_c,0,xilF,2,   zefF, 1,   mueF, 1,mufF, 1)));
  coeff0 := plus_c (coeff0,times_c(complex(  6,0),powers_prod_c(mueB, 1,xilF,2,   zefF, 1,   mueF, 1,mufF, 1)));


    {+  3 mufF   xilF^2 zefF mueF^2 - 4 mufF^2 xilF^2 zefF                          }
  coeff0 := plus_c (coeff0,times_c(complex(  3,0),powers_prod_c(mufF, 1,xilF,2,   zefF, 1,   mueF, 2,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex( -4,0),powers_prod_c(mufF, 2,xilF,2,   zefF, 1,   one_c,0,one_c,0)));

    {+ 12 mufF^2 xilF^2 zefF mueB  + 12 mufF^2 xilF^2 zefF mueF                     }
  coeff0 := plus_c (coeff0,times_c(complex( 12,0),powers_prod_c(mufF, 2,xilF,2,   zefF, 1,   mueB, 1,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex( 12,0),powers_prod_c(mufF, 2,xilF,2,   zefF, 1,   mueF, 1,one_c,0)));

    {+ 10 mufF^3 xilF^2 zefF  + mufF xilF^4 zefF                                   }
  coeff0 := plus_c (coeff0,times_c(complex( 10,0),powers_prod_c(mufF, 3,xilF,2,   zefF, 1,   one_c,0,one_c,0)));
  coeff0 := plus_c (coeff0,times_c(complex(  1,0),powers_prod_c(mufF, 1,xilF,4,   zefF, 1,   one_c,0,one_c,0)));


{    -mueB^3 xilF zeeB  + 4 mueB^3 mueF xilF zeeB + 4 mueB^3 mufF xilF zeeB + 4 mueB^4 xilF zeeB
    + mueF^2 xilF zeeF - 3 mueB mueF^2 xilF zeeF +     6 mueB^2 mueF^2 xilF zeeF
    - 4 mueF^3 xilF zeeF + 16 mueB mueF^3 xilF zeeF + 10 mueF^4 xilF zeeF
    + 2 mueF mufF xilF zeeF - 6 mueB mueF mufF xilF zeeF + 12 mueB^2 mueF mufF xilF zeeF
    - 12 mueF^2 mufF xilF zeeF + 48 mueB mueF^2 mufF xilF zeeF
    + 40 mueF^3 mufF   xilF zeeF
    -  9 mueF   mufF^2 xilF zeeF + 36 mueB mueF mufF^2 xilF zeeF
    + 54 mueF^2 mufF^2 xilF zeeF + 24 mueF mufF^3 xilF zeeF
    + mueF^2 xilF^3 zeeF + 2 mueF mufF xilF^3 zeeF - mueF^2 xilF^3 zeeF^2
    + mufF^2 xilF zefF - 3 mueB mufF^2 xilF zefF + 6 mueB^2 mufF^2 xilF zefF
    - 3 mueF   mufF^2 xilF zefF + 12 mueB mueF mufF^2 xilF zefF
    + 6 mueF^2 mufF^2 xilF zefF -  4 mufF^3           xilF zefF
    + 16 mueB  mufF^3 xilF zefF + 16 mueF mufF^3      xilF zefF
    + 10 mufF^4       xilF zefF
    + mufF^2      xilF^3 zefF - 2 mueF mufF xilF^3 zeeF zefF - mufF^2 xilF^3 zefF^2}


  coeff1 := zero_c;

    {-mueB^3 xilF zeeB  + 4 mueB^3 mueF xilF zeeB + 4 mueB^3 mufF xilF zeeB + 4 mueB^4 xilF zeeB    }
  coeff1 := plus_c (coeff1,times_c(complex( -1,0),powers_prod_c(mueB,3,xilF,1,zeeB,1,one_c,0,one_c,0)));
  coeff1 := plus_c (coeff1,times_c(complex(  4,0),powers_prod_c(mueB,3,xilF,1,zeeB,1, mueF,1,one_c,0)));
  coeff1 := plus_c (coeff1,times_c(complex(  4,0),powers_prod_c(mueB,3,xilF,1,zeeB,1, mufF,1,one_c,0)));
  coeff1 := plus_c (coeff1,times_c(complex(  4,0),powers_prod_c(mueB,4,xilF,1,zeeB,1,one_c,0,one_c,0)));

    {+ mueF^2 xilF zeeF - 3 mueB mueF^2 xilF zeeF +     6 mueB^2 mueF^2 xilF zeeF                   }
  coeff1 := plus_c (coeff1,times_c(complex(  1,0),powers_prod_c(mueF,2,xilF,1,zeeF,1,one_c,0,one_c,0)));
  coeff1 := plus_c (coeff1,times_c(complex( -3,0),powers_prod_c(mueF,2,xilF,1,zeeF,1, mueB,1,one_c,0)));
  coeff1 := plus_c (coeff1,times_c(complex(  6,0),powers_prod_c(mueF,2,xilF,1,zeeF,1, mueB,2,one_c,0)));

    {- 4 mueF^3 xilF zeeF + 16 mueB mueF^3 xilF zeeF + 10 mueF^4 xilF zeeF                          }
  coeff1 := plus_c (coeff1,times_c(complex( -4,0),powers_prod_c(mueF,3,xilF,1,zeeF,1,one_c,0,one_c,0)));
  coeff1 := plus_c (coeff1,times_c(complex( 16,0),powers_prod_c(mueF,3,xilF,1,zeeF,1, mueB,1,one_c,0)));
  coeff1 := plus_c (coeff1,times_c(complex( 10,0),powers_prod_c(mueF,4,xilF,1,zeeF,1,one_c,0,one_c,0)));

    {+ 2 mueF mufF xilF zeeF - 6 mueB mueF mufF xilF zeeF + 12 mueB^2 mueF mufF xilF zeeF           }
  coeff1 := plus_c (coeff1,times_c(complex(  2,0),powers_prod_c(mueF,1,xilF,1,zeeF,1, mufF,1,one_c,0)));
  coeff1 := plus_c (coeff1,times_c(complex( -6,0),powers_prod_c(mueF,1,xilF,1,zeeF,1, mufF,1, mueB,1)));
  coeff1 := plus_c (coeff1,times_c(complex( 12,0),powers_prod_c(mueF,1,xilF,1,zeeF,1, mufF,1, mueB,2)));

    {- 12 mueF^2 mufF xilF zeeF + 48 mueB mueF^2 mufF xilF zeeF                                     }
  coeff1 := plus_c (coeff1,times_c(complex(-12,0),powers_prod_c(mueF,2,xilF,1,zeeF,1, mufF,1,one_c,0)));
  coeff1 := plus_c (coeff1,times_c(complex( 48,0),powers_prod_c(mueF,2,xilF,1,zeeF,1, mufF,1, mueB,1)));

    {+ 40 mueF^3 mufF   xilF zeeF                                                                   }
  coeff1 := plus_c (coeff1,times_c(complex( 40,0),powers_prod_c(mueF,3,xilF,1,zeeF,1, mufF,1,one_c,0)));

    {-  9 mueF   mufF^2 xilF zeeF + 36 mueB mueF mufF^2 xilF zeeF                                   }
  coeff1 := plus_c (coeff1,times_c(complex( -9,0),powers_prod_c(mueF,1,xilF,1,zeeF,1, mufF,2,one_c,0)));
  coeff1 := plus_c (coeff1,times_c(complex( 36,0),powers_prod_c(mueF,1,xilF,1,zeeF,1, mufF,2, mueB,1)));

    {+ 54 mueF^2 mufF^2 xilF zeeF + 24 mueF mufF^3 xilF zeeF                                        }
  coeff1 := plus_c (coeff1,times_c(complex( 54,0),powers_prod_c(mueF,2,xilF,1,zeeF,1, mufF,2,one_c,0)));
  coeff1 := plus_c (coeff1,times_c(complex( 24,0),powers_prod_c(mueF,1,xilF,1,zeeF,1, mufF,3,one_c,0)));

    {+ mueF^2 xilF^3 zeeF + 2 mueF mufF xilF^3 zeeF - mueF^2 xilF^3 zeeF^2                          }
  coeff1 := plus_c (coeff1,times_c(complex(  1,0),powers_prod_c(mueF,2,xilF,3,zeeF,1,one_c,0,one_c,0)));
  coeff1 := plus_c (coeff1,times_c(complex(  2,0),powers_prod_c(mueF,1,xilF,3,zeeF,1, mufF,1,one_c,0)));
  coeff1 := plus_c (coeff1,times_c(complex( -1,0),powers_prod_c(mueF,2,xilF,3,zeeF,2,one_c,0,one_c,0)));

    {+ mufF^2 xilF zefF - 3 mueB mufF^2 xilF zefF + 6 mueB^2 mufF^2 xilF zefF                       }
  coeff1 := plus_c (coeff1,times_c(complex(  1,0),powers_prod_c(mufF,2,xilF,1,zefF,1,one_c,0,one_c,0)));
  coeff1 := plus_c (coeff1,times_c(complex( -3,0),powers_prod_c(mufF,2,xilF,1,zefF,1, mueB,1,one_c,0)));
  coeff1 := plus_c (coeff1,times_c(complex(  6,0),powers_prod_c(mufF,2,xilF,1,zefF,1, mueB,2,one_c,0)));

    {- 3 mueF   mufF^2 xilF zefF + 12 mueB mueF mufF^2 xilF zefF                                    }
  coeff1 := plus_c (coeff1,times_c(complex( -3,0),powers_prod_c(mufF,2,xilF,1,zefF,1, mueF,1,one_c,0)));
  coeff1 := plus_c (coeff1,times_c(complex( 12,0),powers_prod_c(mufF,2,xilF,1,zefF,1, mueF,1, mueB,1)));

    {+ 6 mueF^2 mufF^2 xilF zefF -  4 mufF^3      xilF zefF                                         }
  coeff1 := plus_c (coeff1,times_c(complex(  6,0),powers_prod_c(mufF,2,xilF,1,zefF,1, mueF,2,one_c,0)));
  coeff1 := plus_c (coeff1,times_c(complex( -4,0),powers_prod_c(mufF,3,xilF,1,zefF,1,one_c,0,one_c,0)));

    {+ 16 mueB  mufF^3 xilF zefF + 16 mueF mufF^3   xilF zefF                                       }
  coeff1 := plus_c (coeff1,times_c(complex( 16,0),powers_prod_c(mufF,3,xilF,1,zefF,1, mueB,1,one_c,0)));
  coeff1 := plus_c (coeff1,times_c(complex( 16,0),powers_prod_c(mufF,3,xilF,1,zefF,1, mueF,1,one_c,0)));

    {+ 10 mufF^4        xilF zefF                                                                   }
  coeff1 := plus_c (coeff1,times_c(complex( 10,0),powers_prod_c(mufF,4,xilF,1,zefF,1,one_c,0,one_c,0)));

    {+ mufF^2       xilF^3 zefF - 2 mueF mufF xilF^3 zeeF zefF - mufF^2 xilF^3 zefF^2               }
  coeff1 := plus_c (coeff1,times_c(complex(  1,0),powers_prod_c(mufF,2,xilF,3,zefF,1,one_c,0,one_c,0)));
  coeff1 := plus_c (coeff1,times_c(complex( -2,0),powers_prod_c(mufF,1,xilF,3,zefF,1, zeeF,1, mueF,1)));
  coeff1 := plus_c (coeff1,times_c(complex( -1,0),powers_prod_c(mufF,2,xilF,3,zefF,2,one_c,0,one_c,0)));

  coeff1 := times_c(coeff1,Im_c);
  coeff1 := times_c(coeff1,Pi_c);


{    -mueB^3 zeeB/3 + 4 mueB^4 zeeB/3
    - (10 mueB^5 zeeB)/3 + 4/3 mueB^3 mueF zeeB - 20/3 mueB^4 mueF zeeB -
    10/3 mueB^3 mueF^2 zeeB + 4/3 mueB^3 mufF zeeB -
    20/3 mueB^4 mufF zeeB - 20/3 mueB^3 mueF mufF zeeB -
    10/3 mueB^3 mufF^2 zeeB - (mueF^3 zeeF)/3 +
    4/3 mueB mueF^3 zeeF -
    10/3 mueB^2 mueF^3 zeeF + (4 mueF^4 zeeF)/3 -
    20/3 mueB mueF^4 zeeF - (10 mueF^5 zeeF)/3 -
    mueF^2 mufF zeeF + 4 mueB mueF^2 mufF zeeF -
    10 mueB^2 mueF^2 mufF zeeF + 16/3 mueF^3 mufF zeeF -
    80/3 mueB mueF^3 mufF zeeF - 50/3 mueF^4 mufF zeeF -
    mueF mufF^2 zeeF + 4 mueB mueF mufF^2 zeeF -
    10 mueB^2 mueF mufF^2 zeeF + 8 mueF^2 mufF^2 zeeF -
    40 mueB mueF^2 mufF^2 zeeF - 100/3 mueF^3 mufF^2 zeeF +
    4 mueF mufF^3 zeeF - 20 mueB mueF mufF^3 zeeF -
    30 mueF^2 mufF^3 zeeF - 10 mueF mufF^4 zeeF -
    2/3 mueF^3 xilF^2 zeeF - 2 mueF^2 mufF xilF^2 zeeF -
    2 mueF mufF^2 xilF^2 zeeF + 4/3 mueF^3 xilF^2 zeeF^2 +
    3 mueF^2 mufF xilF^2 zeeF^2 - (mufF^3 zefF)/3 +
    4/3 mueB mufF^3 zefF - 10/3 mueB^2 mufF^3 zefF +
    4/3 mueF mufF^3 zefF - 20/3 mueB mueF mufF^3 zefF -
    10/3 mueF^2 mufF^3 zefF + (4 mufF^4 zefF)/3 -
    20/3 mueB mufF^4 zefF -
    20/3 mueF mufF^4 zefF - (10 mufF^5 zefF)/3 -
    2/3 mufF^3 xilF^2 zefF + mueF^2 mufF xilF^2 zeeF zefF +
    4 mueF mufF^2 xilF^2 zeeF zefF + 4/3 mufF^3 xilF^2 zefF^2 }


{    -mueB^3 zeeB/3 + 4 mueB^4 zeeB/3 - 10 mueB^5 zeeB/3
    + 4/3 mueB^3 mueF zeeB - 20/3 mueB^4 mueF zeeB - 10/3 mueB^3 mueF^2 zeeB
    + 4/3 mueB^3 mufF zeeB -  20/3 mueB^4 mufF zeeB
    - 20/3 mueB^3 mueF mufF zeeB - 10/3 mueB^3 mufF^2 zeeB
    - mueF^3 zeeF/3 + 4/3 mueB mueF^3 zeeF -  10/3 mueB^2 mueF^3 zeeF
    + 4 mueF^4 zeeF/3
    - 20/3 mueB mueF^4 zeeF - 10 mueF^5 zeeF/3
    - mueF^2 mufF zeeF + 4 mueB mueF^2 mufF zeeF - 10 mueB^2 mueF^2 mufF zeeF
    + 16/3 mueF^3 mufF zeeF - 80/3 mueB mueF^3 mufF zeeF
    - 50/3 mueF^4 mufF zeeF
    - mueF mufF^2 zeeF + 4 mueB mueF mufF^2 zeeF - 10 mueB^2 mueF mufF^2 zeeF + 8 mueF^2 mufF^2 zeeF
    - 40 mueB mueF^2 mufF^2 zeeF - 100/3 mueF^3 mufF^2 zeeF
    + 4 mueF mufF^3 zeeF - 20 mueB mueF mufF^3 zeeF - 30 mueF^2 mufF^3 zeeF
    - 10 mueF mufF^4 zeeF
    - 2/3 mueF^3 xilF^2 zeeF - 2 mueF^2 mufF xilF^2 zeeF - 2 mueF mufF^2 xilF^2 zeeF
    + 4/3 mueF^3 xilF^2 zeeF^2
    + 3 mueF^2 mufF xilF^2 zeeF^2 - (mufF^3 zefF)/3
    + 4/3 mueB mufF^3 zefF - 10/3 mueB^2 mufF^3 zefF
    + 4/3 mueF mufF^3 zefF - 20/3 mueB mueF mufF^3 zefF
    - 10/3 mueF^2 mufF^3 zefF + (4 mufF^4 zefF)/3
    - 20/3 mueB mufF^4 zefF - 20/3 mueF mufF^4 zefF
    - 10/3 mufF^5 zefF
    - 2/3 mufF^3 xilF^2 zefF + mueF^2 mufF xilF^2 zeeF zefF
    + 4 mueF mufF^2 xilF^2 zeeF zefF + 4/3 mufF^3 xilF^2 zefF^2}


  coeff2 := zero_c;
{    -mueB^3 zeeB/3 + 4 mueB^4 zeeB/3 - 10 mueB^5 zeeB/3                                                }
  coeff2 := plus_c (coeff2,times_c(complex( -1/3,0),powers_prod_c(mueB,3,zeeB,1,   one_c,0,   one_c,0,one_c,0)));
  coeff2 := plus_c (coeff2,times_c(complex(  4/3,0),powers_prod_c(mueB,4,zeeB,1,   one_c,0,   one_c,0,one_c,0)));
  coeff2 := plus_c (coeff2,times_c(complex(-10/3,0),powers_prod_c(mueB,5,zeeB,1,   one_c,0,   one_c,0,one_c,0)));

    {+ 4/3 mueB^3 mueF zeeB - 20/3 mueB^4 mueF zeeB - 10/3 mueB^3 mueF^2 zeeB                           }
  coeff2 := plus_c (coeff2,times_c(complex(  4/3,0),powers_prod_c(mueB,3,zeeB,1,    mueF,1,   one_c,0,one_c,0)));
  coeff2 := plus_c (coeff2,times_c(complex(-20/3,0),powers_prod_c(mueB,4,zeeB,1,    mueF,1,   one_c,0,one_c,0)));
  coeff2 := plus_c (coeff2,times_c(complex(-10/3,0),powers_prod_c(mueB,3,zeeB,1,    mueF,2,   one_c,0,one_c,0)));

    {+ 4/3 mueB^3 mufF zeeB -  20/3 mueB^4 mufF zeeB                                                    }
  coeff2 := plus_c (coeff2,times_c(complex(4/3,0), powers_prod_c(mueB,3,zeeB,1,    mufF,1,   one_c,0,one_c,0)));
  coeff2 := plus_c (coeff2,times_c(complex(-20/3,0),powers_prod_c(mueB,4,zeeB,1,    mufF,1,   one_c,0,one_c,0)));

    {- 20/3 mueB^3 mueF mufF zeeB - 10/3 mueB^3 mufF^2 zeeB                                             }
  coeff2 := plus_c (coeff2,times_c(complex(-20/3,0),powers_prod_c(mueB,3,zeeB,1,    mufF,1,    mueF,1,one_c,0)));
  coeff2 := plus_c (coeff2,times_c(complex(-10/3,0),powers_prod_c(mueB,3,zeeB,1,    mufF,2,   one_c,0,one_c,0)));

    {- mueF^3 zeeF/3 + 4/3 mueB mueF^3 zeeF -  10/3 mueB^2 mueF^3 zeeF                                  }
  coeff2 := plus_c (coeff2,times_c(complex(- 1/3,0),powers_prod_c(mueF,3,zeeF,1,   one_c,0,   one_c,0,one_c,0)));
  coeff2 := plus_c (coeff2,times_c(complex(  4/3,0),powers_prod_c(mueF,3,zeeF,1,    mueB,1,   one_c,0,one_c,0)));
  coeff2 := plus_c (coeff2,times_c(complex(-10/3,0),powers_prod_c(mueF,3,zeeF,1,    mueB,2,   one_c,0,one_c,0)));

    {+ 4 mueF^4 zeeF/3                                                                                  }
  coeff2 := plus_c (coeff2,times_c(complex(  4/3,0),powers_prod_c(mueF,4,zeeF,1,   one_c,0,   one_c,0,one_c,0)));

    {- 20/3 mueB mueF^4 zeeF - 10 mueF^5 zeeF/3                                                         }
  coeff2 := plus_c (coeff2,times_c(complex(-20/3,0),powers_prod_c(mueF,4,zeeF,1,    mueB,1,   one_c,0,one_c,0)));
  coeff2 := plus_c (coeff2,times_c(complex(-10/3,0),powers_prod_c(mueF,5,zeeF,1,   one_c,0,   one_c,0,one_c,0)));

    {- mueF^2 mufF zeeF + 4 mueB mueF^2 mufF zeeF - 10 mueB^2 mueF^2 mufF zeeF                          }
  coeff2 := plus_c (coeff2,times_c(complex(   -1,0),powers_prod_c(mueF,2,zeeF,1,    mufF,1,   one_c,0,one_c,0)));
  coeff2 := plus_c (coeff2,times_c(complex(    4,0),powers_prod_c(mueF,2,zeeF,1,    mufF,1,    mueB,1,one_c,0)));
  coeff2 := plus_c (coeff2,times_c(complex(  -10,0),powers_prod_c(mueF,2,zeeF,1,    mufF,1,    mueB,2,one_c,0)));

    {+ 16/3 mueF^3 mufF zeeF - 80/3 mueB mueF^3 mufF zeeF                                               }
  coeff2 := plus_c (coeff2,times_c(complex( 16/3,0),powers_prod_c(mueF,3,zeeF,1,    mufF,1,   one_c,0,one_c,0)));
  coeff2 := plus_c (coeff2,times_c(complex(-80/3,0),powers_prod_c(mueF,3,zeeF,1,    mufF,1,    mueB,1,one_c,0)));

    {- 50/3 mueF^4 mufF zeeF                                                                            }
  coeff2 := plus_c (coeff2,times_c(complex(-50/3,0),powers_prod_c(mueF,4,zeeF,1,    mufF,1,   one_c,0,one_c,0)));

    {- mueF mufF^2 zeeF + 4 mueB mueF mufF^2 zeeF - 10 mueB^2 mueF mufF^2 zeeF + 8 mueF^2 mufF^2 zeeF   }
  coeff2 := plus_c (coeff2,times_c(complex(   -1,0),powers_prod_c(mueF,1,zeeF,1,    mufF,2,   one_c,0,one_c,0)));
  coeff2 := plus_c (coeff2,times_c(complex(    4,0),powers_prod_c(mueF,1,zeeF,1,    mufF,2,    mueB,1,one_c,0)));
  coeff2 := plus_c (coeff2,times_c(complex(  -10,0),powers_prod_c(mueF,1,zeeF,1,    mufF,2,    mueB,3,one_c,0)));
  coeff2 := plus_c (coeff2,times_c(complex(    8,0),powers_prod_c(mueF,2,zeeF,1,    mufF,2,   one_c,0,one_c,0)));

    {- 40 mueB mueF^2 mufF^2 zeeF - 100/3 mueF^3 mufF^2 zeeF                                            }
  coeff2 := plus_c (coeff2,times_c(complex(  -40,0),powers_prod_c(mueF,2,zeeF,1,    mufF,2,    mueB,1,one_c,0)));
  coeff2 := plus_c (coeff2,times_c(complex(-100/3,0),powers_prod_c(mueF,3,zeeF,1,    mufF,2,   one_c,0,one_c,0)));

    {+ 4 mueF mufF^3 zeeF - 20 mueB mueF mufF^3 zeeF - 30 mueF^2 mufF^3 zeeF                            }
  coeff2 := plus_c (coeff2,times_c(complex(    4,0),powers_prod_c(mueF,1,zeeF,1,    mufF,3,   one_c,0,one_c,0)));
  coeff2 := plus_c (coeff2,times_c(complex(  -20,0),powers_prod_c(mueF,1,zeeF,1,    mufF,3,    mueB,1,one_c,0)));
  coeff2 := plus_c (coeff2,times_c(complex(  -30,0),powers_prod_c(mueF,2,zeeF,1,    mufF,3,   one_c,0,one_c,0)));

    {- 10 mueF mufF^4 zeeF                                                                              }
  coeff2 := plus_c (coeff2,times_c(complex(  -10,0),powers_prod_c(mueF,1,zeeF,1,    mufF,4,   one_c,0,one_c,0)));

    {- 2/3 mueF^3 xilF^2 zeeF - 2 mueF^2 mufF xilF^2 zeeF - 2 mueF mufF^2 xilF^2 zeeF                   }
  coeff2 := plus_c (coeff2,times_c(complex( -2/3,0),powers_prod_c(mueF,3,zeeF,1,   one_c,0,   one_c,0, xilF,2)));
  coeff2 := plus_c (coeff2,times_c(complex( -2  ,0),powers_prod_c(mueF,2,zeeF,1,    mufF,1,   one_c,0, xilF,2)));
  coeff2 := plus_c (coeff2,times_c(complex( -2  ,0),powers_prod_c(mueF,1,zeeF,1,    mufF,2,   one_c,0, xilF,2)));

    {+ 4/3 mueF^3 xilF^2 zeeF^2                                                                       }
  coeff2 := plus_c (coeff2,times_c(complex(  4/3,0),powers_prod_c(mueF,3,zeeF,2,   one_c,0,   one_c,0, xilF,2)));

    {+3 mueF^2 mufF xilF^2 zeeF^2 - (mufF^3 zefF)/3                                                    }
  coeff2 := plus_c (coeff2,times_c(complex(    3,0),powers_prod_c(mueF,2,zeeF,2,    mufF,1,   one_c,0, xilF,2)));
  coeff2 := plus_c (coeff2,times_c(complex( -1/3,0),powers_prod_c(one_c,0,zefF,1,    mufF,3,   one_c,0,one_c,0)));

    {+4/3 mueB mufF^3 zefF - 10/3 mueB^2 mufF^3 zefF                                                  }
  coeff2 := plus_c (coeff2,times_c(complex(  4/3,0),powers_prod_c(mueB,1,zefF,1,    mufF,3,   one_c,0,one_c,0)));
  coeff2 := plus_c (coeff2,times_c(complex(-10/3,0),powers_prod_c(mueB,2,zefF,1,    mufF,3,   one_c,0,one_c,0)));

    {+4/3 mueF mufF^3 zefF - 20/3 mueB mueF mufF^3 zefF                                                }
  coeff2 := plus_c (coeff2,times_c(complex(  4/3,0),powers_prod_c(mueF,1,zefF,1,    mufF,3,   one_c,0,one_c,0)));
  coeff2 := plus_c (coeff2,times_c(complex(-20/3,0),powers_prod_c(mueF,1,zefF,1,    mufF,3,    mueB,1,one_c,0)));

    {-10/3 mueF^2 mufF^3 zefF + (4 mufF^4 zefF)/3                                                      }
  coeff2 := plus_c (coeff2,times_c(complex(-10/3,0),powers_prod_c(mueF,2,zefF,1,    mufF,3,   one_c,0,one_c,0)));
  coeff2 := plus_c (coeff2,times_c(complex(  4/3,0),powers_prod_c(one_c,0,zefF,1,    mufF,4,   one_c,0,one_c,0)));

    {-20/3 mueB mufF^4 zefF - 20/3 mueF mufF^4 zefF                                                      }
  coeff2 := plus_c (coeff2,times_c(complex(-20/3,0),powers_prod_c(mueB,1,zefF,1,    mufF,4,   one_c,0,one_c,0)));
  coeff2 := plus_c (coeff2,times_c(complex(-20/3,0),powers_prod_c(mueF,1,zefF,1,    mufF,4,   one_c,0,one_c,0)));

    {- 10/3 mufF^5 zefF                                                                                 }
  coeff2 := plus_c (coeff2,times_c(complex(-10/3,0),powers_prod_c(one_c,0,zefF,1,    mufF,5,   one_c,0,one_c,0)));

    {- 2/3 mufF^3 xilF^2 zefF + mueF^2 mufF xilF^2 zeeF zefF                                            }
  coeff2 := plus_c (coeff2,times_c(complex( -2/3,0),powers_prod_c(one_c,0,zefF,1,    mufF,3,   one_c,0, xilF,2)));
  coeff2 := plus_c (coeff2,times_c(complex(    1,0),powers_prod_c(mueF,2,zefF,1,    mufF,1,    zeeF,1, xilF,2)));

    {+ 4 mueF mufF^2 xilF^2 zeeF zefF + 4/3 mufF^3 xilF^2 zefF^2                                       }
  coeff2 := plus_c (coeff2,times_c(complex(    4,0),powers_prod_c(mueF,1,zefF,1,    mufF,2,    zeeF,1, xilF,2)));
  coeff2 := plus_c (coeff2,times_c(complex(  4/3,0),powers_prod_c(one_c,0,zefF,2,    mufF,3,   one_c,0, xilF,2)));

  coeff2 := times_c(coeff2,Pi_c);
  coeff2 := times_c(coeff2,Pi_c);

{!((-(1/3)) mueF^4 xilF zeeF - 4/3 mueF^3 mufF xilF zeeF -
    2 mueF^2 mufF^2 xilF zeeF - 4/3 mueF mufF^3 xilF zeeF +
    2/3 mueF^4 xilF zeeF^2 + 8/3 mueF^3 mufF xilF zeeF^2 +
    3 mueF^2 mufF^2 xilF zeeF^2 - 1/3 mufF^4 xilF zefF +
    mueF^2 mufF^2 xilF zeeF zefF +
    8/3 mueF mufF^3 xilF zeeF zefF + 2/3 mufF^4 xilF zefF^2)}

    {- 1/3 mueF^4 xilF zeeF - 4/3 mueF^3 mufF xilF zeeF - 2 mueF^2 mufF^2 xilF zeeF - 4/3 mueF mufF^3 xilF zeeF
    + 2/3 mueF^4 xilF zeeF^2 + 8/3 mueF^3 mufF xilF zeeF^2 +   3 mueF^2 mufF^2 xilF zeeF^2 - 1/3 mufF^4 xilF zefF
    +  mueF^2 mufF^2 xilF zeeF zefF + 8/3 mueF mufF^3 xilF zeeF zefF
    + 2/3 mufF^4 xilF zefF^2}

    {- 1/3 mueF^4 xilF zeeF - 4/3 mueF^3 mufF xilF zeeF - 2 mueF^2 mufF^2 xilF zeeF - 4/3 mueF mufF^3 xilF zeeF     }
    {+ 2/3 mueF^4 xilF zeeF^2 + 8/3 mueF^3 mufF xilF zeeF^2 +   3 mueF^2 mufF^2 xilF zeeF^2 - 1/3 mufF^4 xilF zefF  }
    {+  mueF^2 mufF^2 xilF zeeF zefF + 8/3 mueF mufF^3 xilF zeeF zefF                                               }
    {+ 2/3 mufF^4 xilF zefF^2                                                                                       }


  coeff3 := zero_c;
    {- 1/3 mueF^4 xilF zeeF - 4/3 mueF^3 mufF xilF zeeF - 2 mueF^2 mufF^2 xilF zeeF - 4/3 mueF mufF^3 xilF zeeF     }
  coeff3 := plus_c (coeff3,times_c(complex(  -1/3,0),powers_prod_c(mueF,4,xilF,1,zeeF,1,  one_c,0,one_c,0)));
  coeff3 := plus_c (coeff3,times_c(complex(  -4/3,0),powers_prod_c(mueF,3,xilF,1,zeeF,1,   mufF,1,one_c,0)));
  coeff3 := plus_c (coeff3,times_c(complex(    -2,0),powers_prod_c(mueF,2,xilF,1,zeeF,1,   mufF,2,one_c,0)));
  coeff3 := plus_c (coeff3,times_c(complex(  -4/3,0),powers_prod_c(mueF,1,xilF,1,zeeF,1,   mufF,3,one_c,0)));

    {+ 2/3 mueF^4 xilF zeeF^2 + 8/3 mueF^3 mufF xilF zeeF^2 +   3 mueF^2 mufF^2 xilF zeeF^2 - 1/3 mufF^4 xilF zefF  }
  coeff3 := plus_c (coeff3,times_c(complex(   2/3,0),powers_prod_c(mueF,4,xilF,1,zeeF,2,  one_c,0,one_c,0)));
  coeff3 := plus_c (coeff3,times_c(complex(   8/3,0),powers_prod_c(mueF,3,xilF,1,zeeF,2,   mufF,1,one_c,0)));
  coeff3 := plus_c (coeff3,times_c(complex(     3,0),powers_prod_c(mueF,2,xilF,1,zeeF,2,   mufF,2,one_c,0)));
  coeff3 := plus_c (coeff3,times_c(complex(  -1/3,0),powers_prod_c(one_c,0,xilF,1,zefF,2,   mufF,4,one_c,0)));

    {+  mueF^2 mufF^2 xilF zeeF zefF + 8/3 mueF mufF^3 xilF zeeF zefF                                               }
  coeff3 := plus_c (coeff3,times_c(complex(     1,0),powers_prod_c(mueF,2,xilF,1,zefF,1,   mufF,2, zeeF,1)));
  coeff3 := plus_c (coeff3,times_c(complex(   8/3,0),powers_prod_c(mueF,1,xilF,1,zefF,1,   mufF,3, zeeF,1)));

    {+ 2/3 mufF^4 xilF zefF^2                                                                                      }
  coeff3 := plus_c (coeff3,times_c(complex(   2/3,0),powers_prod_c(one_c,0,xilF,1,zefF,2,   mufF,4,one_c,0)));

  coeff3 := times_c(coeff3,Pi_c);
  coeff3 := times_c(coeff3,Pi_c);
  coeff3 := times_c(coeff3,Im_c);

{!((mueB^5 zeeB)/15 - (2 mueB^5 zeeB^2)/15 + (mueF^5
zeeF)/15 + 1/3 mueF^4 mufF zeeF + 2/3 mueF^3 mufF^2 zeeF +
    2/3 mueF^2 mufF^3 zeeF +
    1/3 mueF mufF^4 zeeF - (2 mueF^5 zeeF^2)/15 -
    2/3 mueF^4 mufF zeeF^2 - 4/3 mueF^3 mufF^2 zeeF^2 -
    mueF^2 mufF^3 zeeF^2 + (mufF^5 zefF)/15 -
    1/3 mueF^2 mufF^3 zeeF zefF -
    2/3 mueF mufF^4 zeeF zefF - (2 mufF^5 zefF^2)/15)}

    {+ 1/15 mueB^5 zeeB - 2/15 mueB^5 zeeB^2                                                                             }
    {+ mueF^5 zeeF/15 + 1/3 mueF^4 mufF zeeF + 2/3 mueF^3 mufF^2 zeeF + 2/3 mueF^2 mufF^3 zeeF + 1/3 mueF mufF^4 zeeF    }
    {- (2 mueF^5 zeeF^2)/15 - 2/3 mueF^4 mufF zeeF^2 - 4/3 mueF^3 mufF^2 zeeF^2 - mueF^2 mufF^3 zeeF^2                   }
    {+ (mufF^5 zefF)/15                                                                                                  }
    {- 1/3 mueF^2 mufF^3 zeeF zefF - 2/3 mueF mufF^4 zeeF zefF                                                           }
    {- (2 mufF^5 zefF^2)/15                                                                                              }


  coeff4 := zero_c;
    {+ 1/15 mueB^5 zeeB - 2/15 mueB^5 zeeB^2                                                                             }
  coeff4 := plus_c (coeff4,times_c(complex(  1/15,0),powers_prod_c(mueB,5,zeeB,1,   one_c,0,   one_c,0,one_c,0)));
  coeff4 := plus_c (coeff4,times_c(complex(- 2/15,0),powers_prod_c(mueB,5,zeeB,2,   one_c,0,   one_c,0,one_c,0)));

    {+ mueF^5 zeeF/15 + 1/3 mueF^4 mufF zeeF + 2/3 mueF^3 mufF^2 zeeF + 2/3 mueF^2 mufF^3 zeeF + 1/3 mueF mufF^4 zeeF    }
  coeff4 := plus_c (coeff4,times_c(complex(  1/15,0),powers_prod_c(mueF,5,zeeF,1,   one_c,0,   one_c,0,one_c,0)));
  coeff4 := plus_c (coeff4,times_c(complex(  1/ 3,0),powers_prod_c(mueF,4,zeeF,1,    mufF,1,   one_c,0,one_c,0)));
  coeff4 := plus_c (coeff4,times_c(complex(  2/ 3,0),powers_prod_c(mueF,3,zeeF,1,    mufF,2,   one_c,0,one_c,0)));
  coeff4 := plus_c (coeff4,times_c(complex(  2/ 3,0),powers_prod_c(mueF,2,zeeF,1,    mufF,3,   one_c,0,one_c,0)));
  coeff4 := plus_c (coeff4,times_c(complex(  1/ 3,0),powers_prod_c(mueF,1,zeeF,1,    mufF,4,   one_c,0,one_c,0)));

    {- (2 mueF^5 zeeF^2)/15 - 2/3 mueF^4 mufF zeeF^2 - 4/3 mueF^3 mufF^2 zeeF^2 - mueF^2 mufF^3 zeeF^2                   }
  coeff4 := plus_c (coeff4,times_c(complex( -2/15,0),powers_prod_c(mueF,5,zeeF,2,   one_c,0,   one_c,0,one_c,0)));
  coeff4 := plus_c (coeff4,times_c(complex( -2/ 3,0),powers_prod_c(mueF,4,zeeF,2,    mufF,1,   one_c,0,one_c,0)));
  coeff4 := plus_c (coeff4,times_c(complex( -4/ 3,0),powers_prod_c(mueF,3,zeeF,2,    mufF,2,   one_c,0,one_c,0)));
  coeff4 := plus_c (coeff4,times_c(complex(    -1,0),powers_prod_c(mueF,2,zeeF,2,    mufF,3,   one_c,0,one_c,0)));

    {+ (mufF^5 zefF)/15                                                                                                  }
  coeff4 := plus_c (coeff4,times_c(complex(  1/15,0),powers_prod_c(mufF,5,zefF,1,   one_c,0,   one_c,0,one_c,0)));

    {- 1/3 mueF^2 mufF^3 zeeF zefF - 2/3 mueF mufF^4 zeeF zefF                                                           }
  coeff4 := plus_c (coeff4,times_c(complex(  -1/3,0),powers_prod_c(mueF,2,zefF,1,    mufF,3,    zeeF,1,one_c,0)));
  coeff4 := plus_c (coeff4,times_c(complex(  -2/3,0),powers_prod_c(mueF,1,zefF,1,    mufF,4,    zeeF,1,one_c,0)));

    {- (2 mufF^5 zefF^2)/15                                                                                              }
  coeff4 := plus_c (coeff4,times_c(complex( -2/15,0),powers_prod_c(one_c,0,zefF,2,    mufF,5,   one_c,0,one_c,0)));

  coeff4 := times_c(coeff4,Pi_c);
  coeff4 := times_c(coeff4,Pi_c);
  coeff4 := times_c(coeff4,Pi_c);
  coeff4 := times_c(coeff4,Pi_c);

  dfdf := zero_c;
  dfdf := plus_c(dfdf,Div_c  (coeffm1,n_c,Errorcode));
  dfdf := plus_c(dfdf,        coeff0       );
  dfdf := plus_c(dfdf,times_c(coeff1, n_c));
  dfdf := plus_c(dfdf,times_c(coeff2, Power_c(n_c,2)));
  dfdf := plus_c(dfdf,times_c(coeff3, Power_c(n_c,3)));
  dfdf := plus_c(dfdf,times_c(coeff4, Power_c(n_c,4)));

  coeff_Rou := xilF;
  coeff_Rou := plus_c (coeff_Rou,times_c(complex(1/3,0),powers_prod_c(xilF,3,one_c,0,one_c,0,   one_c,0,one_c,0)));
  coeff_Rou := plus_c (coeff_Rou,times_c(complex(1/5,0),powers_prod_c(xilF,5,one_c,0,one_c,0,   one_c,0,one_c,0)));
  coeff_no_Rou := xilF_no_Rou;
  coeff_no_Rou := plus_c (coeff_no_Rou,times_c(complex(1/3,0),powers_prod_c(xilF_no_Rou,3,one_c,0,one_c,0,   one_c,0,one_c,0)));
  coeff_no_Rou := plus_c (coeff_no_Rou,times_c(complex(1/5,0),powers_prod_c(xilF_no_Rou,5,one_c,0,one_c,0,   one_c,0,one_c,0)));
  dfdf_Rou:= minus_c(coeff_Rou,coeff_no_Rou);
  dfdf_Rou := times_c(dfdf_Rou,Im_c);
  dfdf_Rou := div_c  (dfdf_Rou,Pi_c,Errorcode);
  dfdf_Rou := div_c  (dfdf_Rou,n_c,Errorcode);
end;

procedure TPert.dfstar_Pe5(var df_Pert,dG_Pert,df_Rou,dG_Rou : Double; var Errorcode:Byte);
var df_Pert_raw,dG_Pert_raw,df_Pert_ref,dG_Pert_ref : Double;
    dfdf,dfdf_Rou_Raw,dfdf_Rou_Ref : TComplex;
begin
  dfdf_Pe5(Pars,dfdf,dfdf_Rou_Raw,Errorcode);
  df_Pert_raw := dfdf.real * n_r * f_fund;
  dG_Pert_raw := dfdf.imag * n_r * f_fund;
  dfdf_Pe5(Ref_Pars,dfdf,dfdf_Rou_Ref,Errorcode);
  df_Pert_ref := dfdf.real * n_r * f_fund;
  dG_Pert_ref := dfdf.imag * n_r * f_fund;
  df_Pert := df_Pert_raw - df_Pert_ref;
  dG_Pert := dG_Pert_raw - dG_Pert_ref;
  df_Rou := (dfdf_Rou_Raw.real - dfdf_Rou_Ref.real) * n_r * f_fund;
  dG_Rou := (dfdf_Rou_Raw.imag - dfdf_Rou_Ref.imag) * n_r * f_fund;
end;

end.
