unit QTM_SLA;
interface
uses Windows,Dialogs,Messages,Controls,StdCtrls,SysUtils,Graphics,QTM_Decl,QTM_Complex,QTM_Main,Math;

type TSLA = Class(Tobject)
  function Jc(i_layer: integer; n_r_loc : Double; GPars : TPars) : TComplex;
  procedure x_to_Pars(x : Double);
  procedure a_to_Pars(a: TDouble_NP);
  function Fresnel_Refl(B_ref : Boolean; var Errorcode: Byte; var ZL_rough : TComplex) : TComplex;
  function ZL_Fresnel(B_ref : Boolean; var Errorcode: Byte; var ZL_rough: Tcomplex) : TComplex;
  function ZL_Rou_Sh_Sma(Zliq : TComplex; hr, asprat, rho, om : Double) : TComplex;
  procedure dfstar_from_Z(var df_loc,dG_loc,df_rou,dG_rou : Double; var Errorcode: Byte);
  procedure Convert_VE_Pars(i_layer,i_ve_old, i_ve_new: Integer; Pars_old : TPars; var Pars_new : TPars);
  procedure Vals_User_to_JpJpp(i_layer,i_ve_old: Integer; User_Pars : TPars; var Pars_JpJpp : TPars);
  procedure Vals_JpJpp_to_User(i_layer,i_ve_new: Integer; Pars_JpJpp : TPars; var User_Pars : TPars);
  procedure Exponents_User_to_JpJpp(i_layer,i_ve     : Integer; User_Pars : TPars; var Pars_JpJpp : TPars);
  procedure Exponents_JpJpp_to_User(i_layer,i_ve_new : Integer; Pars_JpJpp : TPars; var User_Pars : TPars);
  procedure Calc_Slopes(Jp_of_n, Jpp_of_n : TDouble_nhm; var slope_p,slope_pp : Double);
  procedure JpJpp_Infinite_Warning(i_layer : integer; var Pars_JpJpp : TPars);
  private
    impratio,ZL           : TComplex;
    propinterf,proplayer  : T22Matrix_C;
    ampl,ampl_inter       : T2Vector_C;
    thickn,rho            : array[1..nlayer_max] of Double;
    kc,Zc                 : array[1..nlayer_max] of TComplex;
end;

var SLA : TSLA;

implementation
uses QTM_MIO;

function TSLA.Jc(i_layer: integer; n_r_loc : Double; GPars : TPars) : TComplex;
var
  p,pp,Jp_cen, Jpp_cen, p_exp,pp_exp, Gp_of_om, Gpp_of_om, om, Gsqr, Gabs, n_norm,
  tandel,cosdel, sindel, tandelinv, cosdelinv, sindelinv, delinv : Double;
  i_ve : integer;
begin
  om := 2*Pi*f_fund * n_r_loc;
  n_norm := f_fund * n_r_loc / f_Jref;
  case i_layer of
    1 : begin p := GPars.Jp_e;  pp := GPars.Jpp_e;  p_exp := GPars.Jp_exp_e;  pp_exp := GPars.Jpp_exp_e; i_ve := GPars.i_ve_e; end;
    2 : begin p := GPars.Jp_f;  pp := GPars.Jpp_f;  p_exp := GPars.Jp_exp_f;  pp_exp := GPars.Jpp_exp_f; i_ve := GPars.i_ve_f; end;
    3 : begin p := GPars.Jp_b;  pp := GPars.Jpp_b;  p_exp := GPars.Jp_exp_b;  pp_exp := GPars.Jpp_exp_b; i_ve := GPars.i_ve_b; end;
  end;
  case i_ve of
    1 : begin
      Jp_cen := p; Jpp_cen := pp;
      Jc.real :=  Jp_cen  * 1e-6 * power(n_norm,p_exp);
      Jc.imag := -Jpp_cen * 1e-6 * power(n_norm,pp_exp);
    end;
    2 : begin
      Gp_of_om := p * power(n_norm,p_exp);
      Gpp_of_om := pp * power(n_norm,pp_exp);
      Gsqr := sqr(Gp_of_om) + sqr(Gpp_of_om);
      Jc.real :=   Gp_of_om / Gsqr  * 1e-6;
      Jc.imag := -Gpp_of_om / Gsqr  * 1e-6;
    end;
    3 : begin
      Gp_of_om := pp*1e-3/1e6 * om * power(n_norm,pp_exp);
      Gpp_of_om := p*1e-3/1e6 * om * power(n_norm,p_exp);
      Gsqr := sqr(Gp_of_om) + sqr(Gpp_of_om);
      Jc.real :=   Gp_of_om / Gsqr  * 1e-6;
      Jc.imag := -Gpp_of_om / Gsqr  * 1e-6;
    end;
    4 : begin
      Gp_of_om := p * power(n_norm,p_exp);
      Gpp_of_om := pp*1e-3/1e6 * om * power(n_norm,pp_exp);
      Gsqr := sqr(Gp_of_om) + sqr(Gpp_of_om);
      Jc.real :=   Gp_of_om / Gsqr  * 1e-6;
      Jc.imag := -Gpp_of_om / Gsqr  * 1e-6;
    end;
    5 : begin
      tandel := pp;
      cosdel := 1./sqrt(1.+ sqr(tandel));
      sindel := tandel * cosdel;
      Jp_cen := p * cosdel; Jpp_cen := p * sindel;
      Jc.real :=  Jp_cen  * 1e-6 * power(n_norm,p_exp);
      Jc.imag := -Jpp_cen * 1e-6 * power(n_norm,pp_exp);
    end;
    6 : begin
      Gabs := p;
      tandel := pp;
      cosdel := 1./sqrt(1.+ sqr(tandel));
      sindel := tandel * cosdel;
      Gp_of_om  := Gabs * cosdel * power(n_norm,p_exp);
      Gpp_of_om := Gabs * sindel * power(n_norm,pp_exp);
      Gsqr := sqr(Gp_of_om) + sqr(Gpp_of_om);
      Jc.real :=   Gp_of_om / Gsqr  * 1e-6;
      Jc.imag := -Gpp_of_om / Gsqr  * 1e-6;
    end;
    7 : begin
      tandelinv := pp;
      if tandelinv > 1e-22 then begin
        tandel := 1 / tandelinv;
        cosdel := 1./sqrt(1.+ sqr(tandel));
        sindel := tandel * cosdel;
      end else begin
        cosdel := 0;
        sindel := 1;
      end;
      Gabs := p*1e-3*om/1e6;
      Gp_of_om  := Gabs * cosdel * power(n_norm,p_exp);
      Gpp_of_om := Gabs * sindel * power(n_norm,pp_exp);
      Gsqr := sqr(Gp_of_om) + sqr(Gpp_of_om);
      Jc.real :=   Gp_of_om / Gsqr  * 1e-6;
      Jc.imag := -Gpp_of_om / Gsqr  * 1e-6;
    end;
  end;
end;

procedure TSLA.a_to_Pars(a: TDouble_NP);
var i : integer;
begin
  for i := 1 to n_fit_pars do begin
    case fit_pointer[i] of
      t_e_ID                 : Pars.t_e  := a[i];
      t_f_ID                 : Pars.t_f  := a[i];

      rho_e_ID               : Pars.rho_e  := a[i];
      rho_f_ID               : Pars.rho_f  := a[i];
      rho_b_ID               : Pars.rho_b  := a[i];

      Jp_e_ID                : Pars.Jp_e  := a[i];
      Jp_f_ID                : Pars.Jp_f  := a[i];
      Jp_b_ID                : Pars.Jp_b  := a[i];

      Jp_exp_e_ID            : Pars.Jp_exp_e  := a[i];
      Jp_exp_f_ID            : Pars.Jp_exp_f  := a[i];
      Jp_exp_b_ID            : Pars.Jp_exp_b  := a[i];

      Jpp_e_ID               : Pars.Jpp_e  := a[i];
      Jpp_f_ID               : Pars.Jpp_f  := a[i];
      Jpp_b_ID               : Pars.Jpp_b  := a[i];

      Jpp_exp_e_ID           : Pars.Jpp_exp_e  := a[i];
      Jpp_exp_f_ID           : Pars.Jpp_exp_f  := a[i];
      Jpp_exp_b_ID           : Pars.Jpp_exp_b := a[i];

      vers_ID                : Pars.vers := a[i];
      AspRat_ID              : Pars.AspRat := a[i];
    end;
  end;
end;

procedure TSLA.x_to_Pars(x : Double);
begin
  case i_rbtn of
    t_e_ID               : Pars.t_e := x;
    t_f_ID               : Pars.t_f := x;
    rho_e_ID             : Pars.rho_e := x;
    rho_f_ID             : Pars.rho_f := x;
    rho_b_ID             : Pars.rho_b := x;
    Jp_e_ID              : Pars.Jp_e := x;
    Jp_f_ID              : Pars.Jp_f := x;
    Jp_b_ID              : Pars.Jp_b := x;
    Jp_exp_e_ID          : Pars.Jp_exp_e := x;
    Jp_exp_f_ID          : Pars.Jp_exp_f := x;
    Jp_exp_b_ID          : Pars.Jp_exp_b := x;
    Jpp_e_ID             : Pars.Jpp_e := x;
    Jpp_f_ID             : Pars.Jpp_f := x;
    Jpp_b_ID             : Pars.Jpp_b := x;
    Jpp_exp_e_ID         : Pars.Jpp_exp_e := x;
    Jpp_exp_f_ID         : Pars.Jpp_exp_f := x;
    Jpp_exp_b_ID         : Pars.Jpp_exp_b := x;
    vers_ID              : Pars.vers := x;
    AspRat_ID            : Pars.AspRat := x;
  end;
end;

function TSLA.Fresnel_Refl(B_ref : Boolean; var Errorcode: Byte; var ZL_rough: Tcomplex) : TComplex;
var ic,i : integer;
    om : double;
    var Zliq_old : TComplex;
    Jc_loc : TComplex;
begin
  om := 2*Pi*n_r*f_fund;
  case B_ref of True : GPars := Ref_Pars; False : GPars := Pars; end;
  thickn[1] := GPars.t_e  * 1e-9; rho[1] := GPars.rho_e  * 1e3;
  thickn[2] := GPars.t_f  * 1e-9; rho[2] := GPars.rho_f  * 1e3;
                                  rho[3] := GPars.rho_b  * 1e3;
  for i := 1 to nlayer_max - 1 do begin
    if (thickn[i] <> 0) then begin
      Jc_loc := Jc(i,n_r,GPars);
      if abs_c(Jc_loc) < 1e-12 then Jc_loc.real := 1e-12;
      Zc[i] := div_c(complex(sqrt(rho[i]),0),sqrt_c(Jc_loc),Errorcode);
      kc[i] := times_c(complex(om * sqrt(rho[i]),0),sqrt_c(Jc_loc));
    end;
  end;
  if (rho[nlayer_max] > 0) then begin
    Jc_loc := Jc(nlayer_max,n_r,GPars);
    if abs_c(Jc_loc) < 1e-12 then Jc_loc.real := 1e-12;
    Zc[nlayer_max] := div_c(complex(sqrt(rho[nlayer_max]),0),sqrt_c(Jc(nlayer_max,n_r,GPars)),Errorcode);
    if GPars.vers > 0 then begin
      Zliq_old := Zc[nlayer_max];
      Zc[nlayer_max] := ZL_Rou_Sh_Sma(Zc[nlayer_max],GPars.vers,GPars.asprat,rho[nlayer_max],om);
      ZL_rough := minus_c(Zc[nlayer_max], Zliq_old);
    end else ZL_rough := complex(0,0);
  end else begin
    Zc[i] := complex(0,0);
    ZL_rough := complex(0,0);
  end;  
  if not(Errorcode = 0) then Exit;

  ampl[1]:= one_c; ampl[2] := complex(0,0);
  for ic := 1 to nlayer_max - 1 do begin
    i := nlayer_max - ic;
    if thickn[i] <> 0 then begin
      if  abs_c(Zc[i]) > 0 then impratio :=  div_c(Zc[i+1],Zc[i],Errorcode);
      if (abs_c(Zc[i]) = 0) and (abs_c(Zc[i+1]) = 0)  then impratio :=  complex(1,0);
      if (abs_c(Zc[i]) = 0) and (abs_c(Zc[i+1]) <> 0) then begin
        //MessageDlg('Acoustic impedance = 0, check layer system',mtInformation,[mbOK],0); -> handled in errorhanddle
        Errorcode := Errorcode or E_SLA_failed;
        Exit;
      end;
      propinterf[1,1] := div_c( plus_c(one_c,impratio),complex(2,0), Errorcode);
      propinterf[1,2] := div_c(minus_c(one_c,impratio),complex(2,0), Errorcode);
      propinterf[2,1] := div_c(minus_c(one_c,impratio),complex(2,0), Errorcode);
      propinterf[2,2] := div_c( plus_c(one_c,impratio),complex(2,0), Errorcode);
      if abs_c(times_c(kc[i],Complex(thickn[i],0))) > 1000 then begin
        Errorcode := Errorcode or E_kd_failed;
        {MessageDlg('An argument of a tangent became too large' + #13 +
                   'Layer is either  too thick or too soft'
                   , mtInformation,[mbOK],0); -> handled in errorhanddle}
        Exit;
      end;
      proplayer[1,1]   := exp_c(times_c(times_c(im_c,kc[i]),complex( thickn[i],0)));
      proplayer[2,2]   := exp_c(times_c(times_c(im_c,kc[i]),complex(-thickn[i],0)));
      proplayer[1,2]   := complex(0,0);
      proplayer[2,1]   := complex(0,0);
      ampl_inter := matrix_times_vector_c(propinterf,ampl);
      ampl       := matrix_times_vector_c(proplayer,ampl_inter);
    end else Zc[i] := Zc[i + 1];
  end;
  if abs_c(ampl[1]) > 0 then Fresnel_Refl := div_c(ampl[2],ampl[1], Errorcode)
  else begin Messagedlg('Infinite reflectivity',mtinformation,[mbOK],0); Exit; end;
end;

function TSLA.ZL_Fresnel(B_ref : Boolean; var Errorcode: Byte; var ZL_rough: Tcomplex) : TComplex;
var reflc : TComplex;
begin
  if not(Errorcode = 0) then Exit;
  reflc := Fresnel_Refl(B_ref, Errorcode,ZL_rough); //only called here
  if not(Errorcode = 0) then Exit;
  if abs_c(plus_c(one_c,reflc)) > 0 then
    ZL_Fresnel := times_c(Zc[1],div_c(minus_c(one_c,reflc),plus_c(one_c,reflc), Errorcode))
  else begin Messagedlg('SLA.ZL_LayS: plus_c(one_c,reflc)) > 0',mtinformation,[mbOK],0); Exit; end;
end;

procedure TSLA.dfstar_from_Z(var df_loc,dG_loc,df_rou,dG_rou : Double; var Errorcode: Byte);
var df_raw,dG_raw,df_ref,dG_ref : Double;
    ZL_rough_raw, ZL_rough_ref : TComplex;
begin
  ZL := ZL_Fresnel(False,Errorcode,ZL_rough_raw);
  if not(errorcode = 0) then exit; //PS
  df_raw := - f_fund / (Pi * Zq) * ZL.imag;
  dG_raw :=   f_fund / (Pi * Zq) * ZL.real;
  ZL := ZL_Fresnel(True,Errorcode,ZL_rough_ref);
  df_ref := - f_fund / (Pi * Zq) * ZL.imag;
  dG_ref :=   f_fund / (Pi * Zq) * ZL.real;
  df_loc := (df_raw - df_ref);
  dG_loc := (dG_raw - dG_ref);
  df_rou := - f_fund / (Pi * Zq) * (ZL_rough_raw.imag - ZL_rough_ref.imag);
  dG_rou :=   f_fund / (Pi * Zq) * (ZL_rough_raw.real - ZL_rough_ref.real);
end;

function TSLA.ZL_Rou_Sh_Sma(Zliq : TComplex; hr, asprat, rho, om : Double) : TComplex;
var k_c_sqr, one_p_hrsqr_kcsqr : TComplex;
    om_rho_sqr, Sauerbrey_term : Double;
begin
  om_rho_sqr := sqr(om) * sqr(rho);
  k_c_sqr := div_c(complex(om_rho_sqr,0) , sqr_c(Zliq));
  one_p_hrsqr_kcsqr := plus_c(complex(1,0), times_c(complex(sqr(hr*1e-9),0),k_c_sqr));
  Sauerbrey_term := om * rho * hr*1e-9 * 3 * sqrt(pi) / 2 * asprat;
  ZL_Rou_Sh_Sma := plus_c( times_c(Zliq,one_p_hrsqr_kcsqr) , complex(0, Sauerbrey_term));
end;

procedure TSLA.Convert_VE_Pars(i_layer,i_ve_old, i_ve_new: Integer; Pars_old : TPars; var Pars_new : TPars);
var Pars_JpJpp : TPars;
begin
  Vals_User_to_JpJpp(i_layer,i_ve_old,Pars_old, Pars_JpJpp);
  Vals_JpJpp_to_User(i_layer,i_ve_new,Pars_JpJpp, Pars_new);
  Exponents_User_to_JpJpp(i_layer,i_ve_old,Pars_old, Pars_JpJpp);
  Exponents_JpJpp_to_User(i_layer,i_ve_new,Pars_JpJpp, Pars_new);
  case i_layer of
    1 : Pars_new.i_ve_e := i_ve_new;
    2 : Pars_new.i_ve_f := i_ve_new;
    3 : Pars_new.i_ve_b := i_ve_new;
  end;  
end;

procedure TSLA.Vals_User_to_JpJpp(i_layer,i_ve_old: Integer; User_Pars : TPars; var Pars_JpJpp : TPars);
var
  om, Gsqr, Gabs, Jp, Jpp, Gp, Gpp, New_Jp, New_Jpp,
  tandel, tandelinv, cosdel, sindel, cosdelinv, sindelinv, Jabs : double;
begin
  move(User_Pars,Pars_JpJpp,Sizeof(TPars));
  Pars_JpJpp.i_ve_e := 1;
  Pars_JpJpp.i_ve_f := 1;
  Pars_JpJpp.i_ve_b := 1;
  om := 2 * pi * f_Jref;
  case i_layer of
    1 : begin Jp := User_Pars.Jp_e; Jpp := User_Pars.Jpp_e; end;
    2 : begin Jp := User_Pars.Jp_f; Jpp := User_Pars.Jpp_f; end;
    3 : begin Jp := User_Pars.Jp_b; Jpp := User_Pars.Jpp_b; end;
  end;
  case i_ve_old of
    1 : begin
      New_Jp   := Jp;
      New_Jpp  := Jpp;
    end;
    2 : begin
      Gsqr := sqr(Jp) + sqr(Jpp);
      if Gsqr > 1e-22 then begin
        New_Jp   := Jp  / Gsqr;
        New_Jpp  := Jpp / Gsqr;
      end else JpJpp_Infinite_Warning(i_layer,Pars_JpJpp);
    end;
    3 : begin
      Gp  := Jpp * om / 1e9;
      Gpp := Jp * om / 1e9;
      Gsqr := (sqr(Gp) + sqr(Gpp));
      if Gsqr > 1e-22 then begin
        New_Jp  := Gp  / Gsqr;
        New_Jpp := Gpp / Gsqr;
      end else JpJpp_Infinite_Warning(i_layer,Pars_JpJpp);
    end;
    4 : begin
      Gp  := Jp;
      Gpp := Jpp * om / 1e9;
      Gsqr := sqr(Gp) + sqr(Gpp);
      if Gsqr > 1e-22 then begin
        New_Jp  := Gp  / Gsqr;
        New_Jpp := Gpp / Gsqr;
      end else JpJpp_Infinite_Warning(i_layer,Pars_JpJpp);
    end;
    5 : begin
      tandel := Jpp;
      if tandel >= 1e22 then begin
        cosdel := 0;
        sindel := 1;
      end else begin
        cosdel := 1./sqrt(1.+ sqr(tandel));
        sindel := tandel * cosdel;
      end;
      Jabs := Jp;
      New_Jp  := Jabs * cosdel;
      New_Jpp := Jabs * sindel;
    end;
    6 : begin
      tandel := Jpp;
      if tandel >= 1e22 then begin
        cosdel := 0;
        sindel := 1;
      end else begin
        cosdel := 1./sqrt(1.+ sqr(tandel));
        sindel := tandel * cosdel;
      end;
      Gabs := Jp;
      if Gabs > 1e-22 then begin
        New_Jp  := cosdel / Gabs;
        New_Jpp := sindel / Gabs;
      end else JpJpp_Infinite_Warning(i_layer,Pars_JpJpp);
    end;
    7 : begin
      tandelinv := Jpp;
      if tandelinv >= 1e22 then begin
        cosdelinv := 0;
        sindelinv := 1;
      end else begin
        cosdelinv := 1./sqrt(1.+ sqr(tandelinv));
        sindelinv := tandelinv * cosdelinv;
      end;
      Gabs := Jp * om / 1e9;
      if Gabs > 1e-22 then begin
        New_Jp  := sindelinv / Gabs;
        New_Jpp := cosdelinv / Gabs;
      end else JpJpp_Infinite_Warning(i_layer,Pars_JpJpp);
    end;
  end;
  case i_layer of
    1 : begin Pars_JpJpp.Jp_e := New_Jp; Pars_JpJpp.Jpp_e := New_Jpp; end;
    2 : begin Pars_JpJpp.Jp_f := New_Jp; Pars_JpJpp.Jpp_f := New_Jpp; end;
    3 : begin Pars_JpJpp.Jp_b := New_Jp; Pars_JpJpp.Jpp_b := New_Jpp; end;
  end;
end;

procedure TSLA.Vals_JpJpp_to_User(i_layer,i_ve_new: Integer; Pars_JpJpp : TPars; var User_Pars : TPars);
var om, Jp, Jpp, J_abs, J_abs_sqr, G_abs, eta_abs, tandel, tandelinv, User_Jp, User_Jpp: double;
    i_ve_e_User, i_ve_f_User, i_ve_b_User : Integer;
begin
  i_ve_e_User := User_Pars.i_ve_e;
  i_ve_f_User := User_Pars.i_ve_f;
  i_ve_b_User := User_Pars.i_ve_b;
  move(Pars_JpJpp,User_Pars,Sizeof(TPars));
  User_Pars.i_ve_e := i_ve_e_User;
  User_Pars.i_ve_f := i_ve_f_User;
  User_Pars.i_ve_b := i_ve_b_User;
  om := 2 * pi * f_Jref;
  case i_layer of
    1 : begin Jp := Pars_JpJpp.Jp_e; Jpp := Pars_JpJpp.Jpp_e; User_Pars.i_ve_e := i_ve_new; end;
    2 : begin Jp := Pars_JpJpp.Jp_f; Jpp := Pars_JpJpp.Jpp_f; User_Pars.i_ve_f := i_ve_new; end;
    3 : begin Jp := Pars_JpJpp.Jp_b; Jpp := Pars_JpJpp.Jpp_b; User_Pars.i_ve_b := i_ve_new; end;
  end;
  J_abs_sqr := sqr(Jp) + sqr(Jpp);
  J_abs     := sqrt(J_abs_sqr);
  if Jp  > 1e-22 then tandel    := Jpp/Jp else tandel    := 1e22;
  if Jpp > 1e-22 then tandelinv := Jp/Jpp else tandelinv := 1e22;
  case i_ve_new of
    1 : begin
      User_Jp  := Jp; User_Jpp := Jpp;
    end;
    2 : begin
      if J_abs_sqr > 1e-22 then begin User_Jp  := Jp  / J_abs_sqr; User_Jpp := Jpp / J_abs_sqr; end
      else begin User_Jp  := GapV; User_Jpp := GapV; end;
    end;
    3 : begin
      if J_abs_sqr > 1e-22 then begin User_Jp  := Jpp / J_abs_sqr / om * 1e9; User_Jpp := Jp  / J_abs_sqr / om * 1e9; end
      else begin User_Jp  := GapV; User_Jpp := GapV; end;
    end;
    4 : begin
      if J_abs_sqr > 1e-22 then begin User_Jp  := Jp / J_abs_sqr; User_Jpp := Jpp / J_abs_sqr / om * 1e9; end
      else begin User_Jp  := GapV; User_Jpp := GapV; end;
    end;
    5 : begin
      User_Jp  := J_abs; User_Jpp := tandel;
    end;
    6 : begin
      if J_abs > 1e-22 then begin G_abs  := 1. / J_abs; User_Jp := G_abs; User_Jpp := tandel; end
      else begin User_Jp  := GapV; User_Jpp := GapV; end;
    end;
    7 : begin
      if J_abs > 1e-22 then begin eta_abs  := 1/J_abs / om  * 1e9; User_Jp  := eta_abs; User_Jpp := tandelinv; end
      else begin User_Jp  := GapV; User_Jpp := GapV; end;
    end;
  end;
  case i_layer of
    1 : begin User_Pars.Jp_e := User_Jp; User_Pars.Jpp_e := User_Jpp; end;
    2 : begin User_Pars.Jp_f := User_Jp; User_Pars.Jpp_f := User_Jpp; end;
    3 : begin User_Pars.Jp_b := User_Jp; User_Pars.Jpp_b := User_Jpp; end;
  end;
end;

procedure TSLA.JpJpp_Infinite_Warning(i_layer : integer; var Pars_JpJpp : TPars);
begin
  if i_layer = 1 then begin
    Pars_JpJpp.Jp_e := 1e9;
    Pars_JpJpp.Jpp_e := 1e9;
    if not B_JpJpp_Zero_Warning then MessageDlg('Compliance of layer 1 was infinite, has been set to 1e9 ',mtwarning,[mbOK],0);
  end;
  if i_layer = 2 then begin
    Pars_JpJpp.Jp_f := 1e9;
    Pars_JpJpp.Jpp_f := 1e9;
    if not B_JpJpp_Zero_Warning then MessageDlg('Compliance of layer 2 was infinite, has been set to 1e9 ',mtwarning,[mbOK],0);
  end;
  if i_layer = 3 then begin
    Pars_JpJpp.Jp_b := 1e9;
    Pars_JpJpp.Jpp_b := 1e9;
    if not B_JpJpp_Zero_Warning then MessageDlg('Compliance of bulk was infinite, has been set to 1e9 ',mtwarning,[mbOK],0);
  end;
  B_JpJpp_Zero_Warning := True;
end;

procedure TSLA.Exponents_JpJpp_to_User(i_layer,i_ve_new: Integer; Pars_JpJpp : TPars; var User_Pars : TPars);
var
  Jp_of_n,Jpp_of_n,VEp_of_n, VEpp_of_n : TDouble_nhm;
  Jc_of_n : array[1..nhm] of TComplex;
  slope_p,slope_pp : Double;
  ih : integer;
  Jp,Jpp,J_abs_sqr,User_Jp,User_Jpp,om : Double;
begin
  for ih := 1 to nhm do begin
    om := 2 * pi * n_arr[ih] * f_fund;
    Jc_of_n[ih]  := SLA.Jc(i_layer,n_arr[ih],Pars_JpJpp);
    Jp_of_n[ih]  := abs(Jc_of_n[ih].real);
    Jpp_of_n[ih] := abs(Jc_of_n[ih].imag);
    Jp := Jp_of_n[ih]; Jpp := Jpp_of_n[ih];
    J_abs_sqr := sqr(Jp) + sqr(Jpp);
    case i_ve_new of
      1,5 : begin
        User_Jp  := Jp; User_Jpp := Jpp;
      end;
      2,6 : begin
        if J_abs_sqr > 1e-22 then begin User_Jp  := Jp  / J_abs_sqr; User_Jpp := Jpp / J_abs_sqr; end
        else begin User_Jp  := GapV; User_Jpp := GapV; end;
      end;
      3,7 : begin
        if J_abs_sqr > 1e-22 then begin User_Jp  := Jpp / J_abs_sqr / om * 1e9; User_Jpp := Jp  / J_abs_sqr / om * 1e9; end
        else begin User_Jp  := GapV; User_Jpp := GapV; end;
      end;
      4 : begin
        if J_abs_sqr > 1e-22 then begin User_Jp  := Jp / J_abs_sqr; User_Jpp := Jpp / J_abs_sqr / om * 1e9; end
        else begin User_Jp  := GapV; User_Jpp := GapV; end;
      end;
    end;
    VEp_of_n[ih] := User_Jp; VEpp_of_n[ih] := User_Jpp;
  end;

  Calc_Slopes(VEp_of_n, VEpp_of_n, slope_p,slope_pp);
  case i_layer of
    1 : begin User_Pars.Jp_exp_e := Slope_p; User_Pars.Jpp_exp_e := Slope_pp; end;
    2 : begin User_Pars.Jp_exp_f := Slope_p; User_Pars.Jpp_exp_f := Slope_pp; end;
    3 : begin User_Pars.Jp_exp_b := Slope_p; User_Pars.Jpp_exp_b := Slope_pp; end;
  end;
end;

procedure TSLA.Exponents_User_to_JpJpp(i_layer,i_ve: Integer; User_Pars : TPars; var Pars_JpJpp : TPars);
var
  Jp_of_n, Jpp_of_n : TDouble_nhm;
  Jc_of_n : array[1..nhm] of TComplex;
  slope_p,slope_pp : Double;
  ih : integer;

begin
  for ih := 1 to nhm do begin
    Jc_of_n[ih] := SLA.Jc(i_layer,n_arr[ih],User_Pars);
    Jp_of_n[ih]  := abs(Jc_of_n[ih].real);
    Jpp_of_n[ih] := abs(Jc_of_n[ih].imag);
  end;
  Calc_Slopes(Jp_of_n, Jpp_of_n, slope_p,slope_pp);
  case i_layer of
    1 : begin Pars_JpJpp.Jp_exp_e := Slope_p; Pars_JpJpp.Jpp_exp_e := Slope_pp; end;
    2 : begin Pars_JpJpp.Jp_exp_f := Slope_p; Pars_JpJpp.Jpp_exp_f := Slope_pp; end;
    3 : begin Pars_JpJpp.Jp_exp_b := Slope_p; Pars_JpJpp.Jpp_exp_b := Slope_pp; end;
  end;
end;

procedure TSLA.Calc_Slopes(Jp_of_n, Jpp_of_n : TDouble_nhm; var slope_p,slope_pp : Double);
var x_avg,yp_avg,ypp_avg,rxyp,rxypp,sxx : double;
      norm,ih,n_ovts_inc : integer;
      logx,logyp,logypp : TDouble_nhm;
      B_p_nonzero, B_pp_nonzero : Boolean;
begin
  x_avg := 0; yp_avg := 0; ypp_avg := 0; norm := 0;
  B_p_nonzero := True;
  B_pp_nonzero := True;
  n_ovts_inc := 0;
  for ih := 1 to nhm do begin
    if B_fit_inc[ih] then n_ovts_inc := n_ovts_inc + 1;
  end;
  for ih := 1 to nhm do begin
    if B_fit_inc[ih] or (n_ovts_inc < 2) then begin
      logx[ih] := log10(n_arr[ih]);
      if Jp_of_n[ih]  > 0 then logyp[ih] := log10(Jp_of_n[ih])
      else B_p_nonzero := False;
      if Jpp_of_n[ih] > 0 then logypp[ih] := log10(Jpp_of_n[ih])
      else B_pp_nonzero := False;
      x_avg := x_avg + logx[ih];
      yp_avg := yp_avg + logyp[ih];
      ypp_avg := ypp_avg + logypp[ih];
      norm := norm + 1;
    end;
  end;
  if norm > 0 then begin
    x_avg := x_avg/norm;
    yp_avg := yp_avg/norm;
    ypp_avg := ypp_avg/norm;
  end;
  rxyp := 0; rxypp := 0; sxx := 0;
  for ih := 1 to nhm do begin
    if B_fit_inc[ih] or (n_ovts_inc < 2) then begin
      rxyp := rxyp + (logx[ih] - x_avg) * (logyp[ih] - yp_avg);
      rxypp := rxypp + (logx[ih] - x_avg) * (logypp[ih] - ypp_avg);
      sxx := sxx + (logx[ih] - x_avg) * (logx[ih] - x_avg);
    end;
  end;
  if B_p_nonzero  then Slope_p  := rxyp/sxx  else Slope_p := 0;
  if B_pp_nonzero then Slope_pp := rxypp/sxx else Slope_pp := 0;
end;


end.
